/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import javax.swing.JOptionPane;

/**
 * Třída popisuje tlakové ztráty při proudění přes armatury
 * Typ: 0-šoupě s kruhovou deskou (obdélník)
 * 1-šoupě s klínovou deskou;  2-ventil s rovným sedlem;
 * 3-ventil se šikmým sedlem;  4-z potrubí do odbočky;
 * 5-z odbočky do potrubí;
 * @author V.Hlubuček
 */
public class Armatura extends Element {

    int typ ;
    static String[] typy = {"šoupě s kruhovou deskou (obdélník)","šoupě s klínovou deskou",
        "ventil s rovným sedlem","ventil se šikmým sedlem",
    "proudění směrem z potrubí do odbočky (přímé potrubí je za odbočkou zaslepeno)",
    "proudění směrem z odbočky do potrubí (přímé potrubí je za odbočkou zaslepeno)"};
    private double prumerVstup;

    public Armatura(Proud pr, int typ, double prumerVstup) {
        super();
        this.pr = pr;
        nazElementu = "";        
        this.typ = (typ >5) ? 0 : typ;  // rozsah 0 až 5
        this.prumerVstup = prumerVstup;
        myInit();
    } 

/**
 * Inicializace elementu armatura
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[6];
        tValue = new String[6];
        lTexty[0] = "Armatura";
        lTexty[1] = "Průměr vstup(m)";
        lTexty[2] = "Typ";
        lTexty[3] = "";
        tValue[1] = "0,05";
        tValue[2] = "1";
        tValue[3] = "";       
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        prumerVstup = doubleFromString(tValue[1]);
        typ = (int)doubleFromString(tValue[2]);
        
    }
    
    
    @Override
    public double tlakZtrata() {
                boolean kapalina;
        double d, gsi, ro, t, p, v;
        d = prumerVstup;
        p = pVstup;
        t = tVstup+TABS;    // absolutní teplota (K)        
        if (pVstup*tVstup <= 0 && pr.isTisk()) {
           JOptionPane.showMessageDialog(null,
              "V elementu " + nazElementu + " pr. (" + pr.cProudu + ") nulový tlak nebo teplota !",
              "Armatura", JOptionPane.WARNING_MESSAGE);            
            return -9999;   // chyba
        }
        kapalina = false;
        if (t < pr.getSmes().sTbv(p)) kapalina = true;  
        if (kapalina) {
            ro = pr.getSmes().sHustL(t);
        } else {
            ro = pr.getSmes().srHustGW(t, p);
        } 
        v = 4 * pr.getHmotPrutok() / ro / Math.PI / d / d;
        switch (typ) {
            case 0:
                gsi = 0.15 + 0.5 * d; break;    // šoupě s kruhovou deskou (obdélník)
            case 1:
                gsi = 0.3 + 0.8 * d; break;     // šoupě s klínovou deskou
            case 2:
                gsi = 4.6 + 8 * d; break;       // ventil s rovným sedlem
            case 3:
                gsi = 3.4 - d / 0.1; break;     // ventil se šikmým sedlem
            case 4:
                gsi = 0.75; break;      // proudění směrem z potrubí do odbočky
            case 5:
                gsi = 1.15; break;      // proudění směrem z odbočky do potrubí
            default: gsi = 0;    
        }
        pVystup = p  -  gsi * v * v * ro / 2;
        tVystup = tVstup;
        rychlVstup = v;
        rychlVystup = v;
        report = "Armatura " + typy[typ]+ " (" + nazElementu + ") \n";
        report +=  " průměr potrubí    (m) " + String.format("%6.4f%n",prumerVstup);
        report +=  " hustota       (kg/m3) " + String.format("%5.2f%n",ro);
        report +=  " teplota bodu varu(°C) " + String.format("%5.1f%n",pr.getSmes().sTbv(p)-TABS);
        report +=  " viskozita      (Pa.s) " + String.format("%6.4g%n",pr.getSmes().sViskL(t));
        report +=  " rychlost        (m/s) " + String.format("%6.3f%n",v);
        report +=  " součinitel odporu (1) " + String.format("%6.4f%n",gsi);
        report +=  " teplota          (°C) " + String.format("%5.1f%n",tVstup);
        report +=  " tlak na vstupu   (Pa) " + String.format("%7.0f%n",pVstup);
        report +=  " tlak na výstupu  (Pa) " + String.format("%7.0f%n",pVystup);
        report +=  " tlaková ztráta   (Pa) " + String.format("%6.1f%n",p - pVystup);        
        return pVstup - pVystup;   // tlaková ztráta         
    }

    @Override
    public String vysledkyElement() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
/**
 * Vrací typ armatury
 * @return typ (int)
 */
    public int getTyp() {
        return typ;
    }
/**
 * Nastavuje typ armatury
 * @param typ 
 */
    public void setTyp(int typ) {
        this.typ = typ;
    }
/**
 * Vrací průměr armatury (m)
 * @return pruměr
 */
    public double getPrumerVstup() {
        return prumerVstup;
    }
/**
 * Nastavuje průměr armatury (m)
 * @param prumerVstup 
 */
    public void setPrumerVstup(double prumerVstup) {
        this.prumerVstup = prumerVstup;
    }    
}
