/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;


import fydat.I_Newt;
import fydat.Numer;
import fydat.NumerException;
import fydatlib.Smes;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import org.jdom2.JDOMException;


/**
 * Třída aparátu průtočného ideálně míchaného reaktoru typu "CSTR".<br> 
 * Spojuje několik proudů reaktantů, může probíhat více reakcí (k).<br>
 * Reakce jsou definovány seznamem přidaných instancí objektů Reakce,<br>
 * který zahrnuje stechiometrické koeficienty i případnou rovnováhu reakce,<br> 
 * kdy koef. produktu je kladný, reaktantu záporný a inertu nulový.<br>
 * Parametry kinetiky reakce:  ko, To, E. Mocninová kinetika v koncentracích (mol/m3)<br>
 * Rychlostní konstanta se počítá jako: k = ko.exp[E(T-To)/R/T/To]<br>
 * Pokud se zadá referenční teplota To <= 0, pak  k= ko.exp[-E/R/T]<br>
 * Třída zahrnuje i výměnu tepla reakční směsi s lázní (koef. prostupu, výměnná plocha),<br>
 * případně lze zvolit i zpětný chladič a provozovat reakci za varu reakční směsi,<br>
 * počítá výstupní koncentrace a parametry
 * @author V.Hlubuček
 */
public class CSTRReaktor extends SmesovacIdeal implements I_Newt {

    protected String VERZE = "1.4";
    protected int nSlozReak, nSloz;   // počet složek reakční směsi a pův. směsi
    protected int nSlozReagujicich;   // počet reagujících složek
    protected int nReakci = 0;        // počet reakcí
    /**
     * Režim reaktoru: 0-izotermní; 1-adiab.; 2-výměna tepla 
     */
    protected int rezim = 0;          // režim reaktoru: 0-izotermní; 1-adiab.; 2-výměna tepla
    protected int itypRovn;           // typ rovnováhy
    protected int[] iRef;             // referenční indx pro konverzi j-té reakce
    protected int[] cislaReak;        // pole s čísly složek rakčního proudu
    protected int[] iPor;             // pořadové indexy reagujících složek
    protected boolean zpetnyChladic;  // existence zpětného chladiče
    /**
     * Pole rychlostních konstant reakcí
     */
    protected double[] k;             // rychlostní konstanty reakcí; 
    protected double objemReaktoru;   // objem reaktoru (m)
    protected double objemPrutok;     // objemový průtok (m3/s)
    protected double teplosmenPlocha; // teplosměnná plocha (m2)
    protected double koefProstupu;    // koeficient prostupu tepla (W/m2/K)
    /**
     * Reakční teplota (K)
     */
    protected double t;               // reakční teplota (K)
    /**
     * Chladicí teplota (K)
     */
    protected double t0;              // chladicí teplota (K)
    protected double tbvPoc;          // bod varu vstupní směsi (°C)
    /**
     * Reakční tlak (Pa)
     */
    protected double p;               // reakční tlak
    protected double hust,hust0;      // hustota reakční směsi
    /**
     * Spec.teplo Cp směsi na vstupu a výstupu (J/kg/K)
     */
    protected double cIn, cOut;       // spec.teplo směsi na vstupu a výstupu (J/kg/K)
    protected double x[];             // pomocné pole
    protected double xx[];            // konverze
    protected double sumKonst;        // suma hmot.zl.neragujících složek
    /**
     * Počáteční, konečná a rovnovážná koncentrace složek
     */
    protected double[] c0,c, cRovn;   // počáteční, konečná a rovnovážná koncentrace složek
    protected double[] xMolPoc;       // počáteční molární zlomky
    protected double[] xHmPoc;        // počáteční hmotové zlomky
    /**
     * V/F poměr objemu V a objemového nástřiku F (s)
     */
    protected double vf;              // poměr objemu V a objemového nástřiku F (s)
    protected Proud reakPr;           // reakční proud
    protected Smes smReak;            // reakční směs
    protected List<Reakce> listReakci = new ArrayList<Reakce>();      // seznam reakcí
    
/**
 * Konstruktor reaktoru
 * @param cApar číslo aparátu (int)
 * @param nazevAparatu textový název reaktoru
 */    
    public CSTRReaktor(int cApar,String nazevAparatu) {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 1;         
        this.cApar = cApar;
        this.nazevAparatu = nazevAparatu;
        myInit();        
    }
/**
 * Bezparametrový konstruktor
 */    
    public CSTRReaktor() {
        super();
        pocetInProudu = 1;
        pocetOutProudu = 1;         
        nazevAparatu = "CSTRReaktor";
        myInit();
    }
/**
 * Inicializace aparátu
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[11];
        tValue = new String[11];
        lTexty[0] = "CSTR reaktor";
        lTexty[1] = "Č.aparátu";
        lTexty[2] = "Název aparátu";
        lTexty[3] = "Počet In proudů";
        lTexty[4] = "Počet Out proudů";
        lTexty[5] = "Objem reaktoru(m3)";
        lTexty[6] = "Teplosměnná plocha(m2)";
        lTexty[7] = "Koeficient prostupu (W/m2/K)";
        lTexty[8] = "Režim reaktoru (0,1,2)";
        lTexty[9] = "Zpětný chladič (0-ne;,1-ano)";
        lTexty[10] = ""; // musí být jako ukončovač
        if(objemReaktoru > 0) {        
            tValue[1] = String.format("%3d",cApar);
            tValue[2] = nazevAparatu;
            tValue[3] = String.format("%2d",pocetInProudu);
            tValue[4] = String.format("%2d",pocetOutProudu);
            tValue[5] = String.format("%5.3f",objemReaktoru);
            tValue[6] = String.format("%4.3f",teplosmenPlocha);
            tValue[7] = String.format("%5.0f",koefProstupu);
            tValue[8] = String.format("%3d",rezim);
            tValue[9] = (zpetnyChladic == true)? "1":"0";            
        } else {
            tValue[1] = "0";
            tValue[2] = "CSTR Reaktor";
            tValue[3] = "1";
            tValue[4] = "1";
            tValue[5] = "0,5";
            tValue[6] = "2,5";
            tValue[7] = "150";
            tValue[8] = "2";
            tValue[9] = "0";                        
        }
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        cApar = (int)doubleFromString(tValue[1]);
        nazevAparatu = tValue[2];
        pocetInProudu = (int)doubleFromString(tValue[3]);
        pocetOutProudu = (int)doubleFromString(tValue[4]);
        objemReaktoru = doubleFromString(tValue[5]);
        teplosmenPlocha = doubleFromString(tValue[6]);
        koefProstupu = doubleFromString(tValue[7]);
        rezim = (int)doubleFromString(tValue[8]);
        if((int)doubleFromString(tValue[9]) == 0) zpetnyChladic = false; else zpetnyChladic = true;        
    }    
    
    
/**
 * Přidá reakci do seznamu reakcí reaktoru
 * @param r objekt reakce
 */    
    public void pridejReakci(Reakce r) {
        listReakci.add(r);
        nReakci++;
    }
    
/**
 * Odebere reakci ze seznamu reakcí reaktoru
 * @param r objekt reakce
 */    
    public void odeberReakci(Reakce r) {
        listReakci.remove(r);
        nReakci--;
    } 

    public boolean jeAparatZadan() {
        boolean neni = false;
        if(this == null) return false;
        if(listReakci == null)  return false;    
        if(objemReaktoru == 0) neni = true;
        if(teplosmenPlocha == 0) neni = true;
        return (!neni);
        
    }
/**
 * implementace abstraktní metody
 */ 
    @Override
    public void spoctiAparat() {
        reakce();
        spocten = true;
    }
/**
 * implementace abstraktní metody
 */ 
    @Override
    public void spoctiAparat(boolean tisk) {
        try {
            if(tisk) reakce(true); else reakce(false);
            spocten = true;
        } catch (NumerException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
        }
    }    
    
/**
 * Metoda vyčíslí průběh reakce v reaktoru a v reakčním (výstupním) proudu<br>
 * poskytne zreaagovanou směs.
 * @throws fydat.NumerException
 */    
    public void reakce()  {
        
        int i, j;
        // kontrola zadání
        usporadatProudy();      // řeší nezadání přes proudy      
        if (proudy == null || proudy.isEmpty()) {
            JOptionPane.showMessageDialog(null,
                "Zadání proudů reaktoru č. " + cApar + "  " + nazevAparatu + "\nnení úplné. Končím!",
                 "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);
            return;
        }
        if(listReakci == null || listReakci.isEmpty()) {
            JOptionPane.showMessageDialog(null,
                "Nebyly zadány žádné reakce u reaktoru č. " + cApar + "  " + nazevAparatu + ".\n Končím!",
                 "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);
            return;
        }
        reakPr = smisitProudy(false);    // poskytne výchozí proud s reaktanty, nezobrazí        
        // vytvořit složky vzniklé reakcemi, pokud ještě nebyly ve směsi obsaženy
        nSlozReak = reakPr.getSmes().getnSloz();
        cislaReak = new int[nSlozReak+5];
//        cislaReak = reakPr.getSmes().getCislaSlozek();
        for (i = 0; i < nSlozReak; i++) cislaReak[i] = reakPr.getSmes().getCisloSlozky(i);
        for (Reakce rr : listReakci) {
            int[] rSl = rr.getiSlozky();    // čísla látek, účastnící se reakce
            for (i = 0; i < rSl.length;i++) {
                if (neniPrvkem(Math.abs(rSl[i]), cislaReak)) {  // není ještě v reakčních složkách
                    if (rr.getiStech()[i] > 0) {                     
                      cislaReak[nSlozReak] = Math.abs(rSl[i]);   // přidání složek produktů 
                      nSlozReak++;
//                        System.out.println("i,slozka, cisReak: "+i+"  "+rSl[i]+"  "+cislaReak[nSlozReak]);
                    } else {
                      JOptionPane.showMessageDialog(null,
                        "Reakční složka č. " + rSl[i] + ";  " + nazevAparatu +
                        "\nnení v zadaných proudech. Končím!",
                         "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);                       
                    }
                }
            }
        }

        nSloz = reakPr.getSmes().getnSloz();  // počet složek dosavadní směsi
        x = new double[nSlozReak];  // deklarace pro pořebný počet složek
        for (i = 0; i < nSloz;i++) {
            x[i] = reakPr.getSmes().getxMol(i);     // složení původní směsi
        }
        for (i = nSloz; i < nSlozReak; i++) {
            x[i] = 0;           // nulová počáteční koncentrace produktů
        }       
        try {
            // vytvoření nové reakční směsi se zahrnutím produktů
            smReak = new Smes(nSlozReak,cislaReak);
            smReak.zadatSloz(false, x);
            reakPr.setSmes(smReak);     // v reakčním proudu nastavena směs před reakcí
            smReak.setNeMistniHlasky(true); // Potlačení zobrazování chybových hlášek v okně
        } catch (JDOMException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);                 
        } catch (IOException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
        }
        xHmPoc = smReak.getxHmot();     // počáteční hmotové zlomky
        xMolPoc = smReak.getxMol();
        // vlastní průběh reakce
        t = reakPr.gettVstup() + reakPr.TABS;   // absolutní teplota
        p = reakPr.getpVstup();
        tbvPoc = smReak.sTbv(p)-TABS;        
        hust = smReak.sHust(t, p);    // hustota reakční směsi
        hust0 = hust;
        cIn = smReak.sCpW(t, p);      // specifické teplo směsi  
        objemPrutok =  reakPr.getHmotPrutok()/hust;
        vf = objemReaktoru/objemPrutok;
        c = new double[nSlozReak];
        c0 = new double[nSlozReak];
        cRovn = new double[nSlozReak];        
        for (i = 0; i < nSlozReak; i++) {
            c0[i] = cKonc(i);       // počáteční koncentrace složek
        }
        c = c0.clone();
        // počet reagujících složek
        boolean jeRovn = false;        
        Reakce.reagSlozky(listReakci, smReak);  // lze volat pomocí statické fce, nemusí být tady programováno
        report += Reakce.vratVysledek();    // získání tisku reportu statickou metodou
        jeRovn = Reakce.jeRovn(listReakci); // zjištění zda je některá reakce se seznamu rovnovážná
        HashSet<Integer> reagSl = new HashSet();    // aby byl seznam jedinečných složek -------
        for (Reakce r : listReakci) {
            report += r.getNazevReakce() + String.format("%nk0= %6.2e  t0(K)= %4.1f  Eakt(kJ/mol)= %5.2f  dH(kJ/mol)= %6.2f%n", 
                    r.getK0(),r.getT0(),r.geteAkt()/1000,r.getdH()/1000);
            if(r.rTyp > 0) {jeRovn = true; itypRovn = r.rTyp; }  // aspoň jedna reakce je rovnovážná
            for (i = 0; i < r.getiKinSloz().length;i++) {
               report += String.format("Sl.kin. = %3d  řád= %4.2f%n", r.getiKinSloz()[i],r.getRadSlozky()[i]);
            }
            for (j = 0; j < r.getiSlozky().length; j++) {
                int jj;
                jj = r.getiSlozky()[j];  
                if(jj != 0) {
                    reagSl.add(jj);  // přidají se jen dosud neexistující a nenulové prvky (Hash set)
                    report += String.format("Sl.reak = %3d     %s%n",jj, reakPr.getSmes().getNazevSlozky(porCisloLatky(Math.abs(jj)))); 
                }
            }
            report += String.format("(Kladně produkty, záporně reaktanty)%n");
        }   // -----------------------------------------------------------------
        // výpočet rovnovážného složení v případě vratných reakcí - nemění složení směsi proudu
        if(jeRovn) {
            int stav;
            Rovnovaha rovn = new Rovnovaha(listReakci,c,itypRovn,reakPr);
            rovn.setXxPoc(9000);    // počáteční hodnota rozahu reakce
            try {
                stav = rovn.spoctiRovnovahu(itypRovn, c, t);
                if(stav > 0) {
                    cRovn = c.clone();  // dosaď rovnovážnou koncentraci
                    report += rovn.report;
                } else {
                    for(j =0; j < nSlozReak; j++) cRovn[j] = 0;
                    report += "Rovnováha singularita nebo divergence! - Zpracuji jako nevratnou.\n";
                }
            } catch (IOException ex) {
                Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
            } catch (NumerException ex) {
                Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        iPor = new int[reagSl.size()];    // pole s indexy neznámých (reagujících složek)
        nSlozReagujicich = reagSl.size();        // počet složek vstupujících do reakcí
        i = 0;
        for (Integer ii : reagSl) {
            iPor[i] = porCisloLatky(Math.abs(ii));
            i++;
        }
        reakPr.xHmot = reakPr.sm.getxHmot();        
        reakPr.xMol = reakPr.sm.getxMol();
        reakPr.nazProudu = "Vstup do " + nazevAparatu;
        reakPr.zUzlu = this;
        reakPr.setVysledek(report);
        proudyIn.clear();       // vynulovat vstupy ze směšovače
        proudyIn.add(reakPr.kopirujProud(reakPr));   // vložení smíšeného proudu reaktantů do vstupního proudu
        // řešení soustavy
//        I_Newt f = this;  // není třeba
        x = new double[nSlozReagujicich];
        double s = 0;
        for (i = 0; i < nSlozReagujicich;i++ ) {
            x[i] = smReak.getxHmot(iPor[i]);
            s += x[i];
//            System.out.println("i,c0,xMolPoc: "+iPor[i]+"  "+c0[iPor[i]]);
        }
        sumKonst = 1 - s;
        int nProm = rezim > 0 ? nReakci+1:nReakci;
        xx = new double[nProm];
        int[] pom = new int[nProm];
        for (i = 0; i < nProm; i++) {  // nastavení neznámé pro Newtona
            pom[i] = i;
            xx[i] = 0.45;   // odhady konverze
        }
        if (nProm > nReakci) xx[nReakci] = 1.0;
        int ii = 0, kk;
        s = 9999;
        iRef = new int[nReakci];
        for (Reakce r:listReakci) {
//              Zjištění referenčních složek pro konverzi reakcí (ty se stech. nejnižší poč. koncentrací)
            for (int k = 0; k < r.getiKinSloz().length;k++) {
                kk = porCisloLatky(r.getiKinSloz()[k]);
                if (smReak.getxHmot(kk)/Math.abs(stechKoef(kk,0)) < s) i = kk; s = smReak.getxHmot(kk)/Math.abs(stechKoef(kk,0));
            }
            iRef[ii] = i;       // obsahuje pořadové číslo referenčníé složky (kolikátá složka)
        }
        int stav = 0;
        try {
            stav = new Numer().Newton(this,nProm, nProm, nProm, pom, xx, 0.00001,0.1);
        } catch (NumerException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (stav == -1) {
            JOptionPane.showMessageDialog(null, "CSTRReaktor - singularita!", "CSTRReaktor", 1);
            report += "*** SINGULARITA! Výsledky nepoužitelné.\n";
        }
        if (stav == -2) {
            JOptionPane.showMessageDialog(null, "CSTRReaktor - divergence!", "CSTRReaktor", 1);
            report += "*** DIVERGENCE! Výsledky nepoužitelné.\n";
        }
        if (stav > 0) report += "Počet iterací = " + stav + "\n";
        reakPr.setpVystup(p);
        if (rezim >0) {
           reakPr.settVystup(xx[rezim >0 ? nReakci: nReakci-1]*reakPr.gettVstup());
        } else{
           reakPr.settVystup(reakPr.gettVstup());
        }
        report += "Výsledky výpočtu reaktoru  (Verze " + getVERZE() + ")\n";
        report += String.format("Hmotový průtok (kg/s) = %7.4f  Objem reaktoru (m3) = %7.3f%n",
                reakPr.getHmotPrutok(), objemReaktoru);
        report += String.format("Objemový průtok (m3/s) = %7.5f%n",objemPrutok);
        report += String.format("Hustota směsi (kg/m3) = %5.1f%n",hust);
        report += "Režim " + ((rezim == 0)? "adiabatický" : ((rezim == 1) ? "adiabatický": "výměna tepla"))+"\n";
        if (rezim == 2) {
            report += String.format("Koef. prostupu (W/m2/K) =%5.0f  Výměnná plocha (m2) = %5.2f%n",
                    koefProstupu,teplosmenPlocha);
        }
        report += "Zpětný chladič: " + ((zpetnyChladic)? "ANO":"NE")  + "\n";
        report += String.format("Bod varu vstupní směsi(°C)= %5.1f%n",tbvPoc);
        report += String.format("Teplota vstup (°C,K)= %4.1f  %4.1f%n"
                + "Teplota výstup(°C,K)= %4.1f  %4.1f%nBod varu výstupní směsi (°C) = %4.1f%nTlak(kPa)= %7.2f%n", 
                reakPr.gettVstup(),reakPr.gettVstup()+TABS,
                reakPr.gettVystup(),reakPr.gettVystup()+TABS,
                smReak.sTbv(p)-TABS,
                reakPr.getpVystup()/1000); 
        ii = 0;
        for (Reakce r: listReakci) {
            report += String.format("Konverze %1d vůči  %s    %7.5f%n",ii+1,reakPr.getSmes().getNazevSlozky(iRef[ii]),xx[0]);
            ii++;
        }
        report += "Složka           Název   c0(mol/m3)   c (mol/m3)   x0 hmot  x hmot   x0 mol   x mol\n";
        for (i = 0; i < nSlozReak; i++) {
            report += String.format(" %3d  %16s   %6.4e   %6.4e  %7.4f  %7.4f  %7.4f  %7.4f%n",
                    reakPr.getSmes().getCisloSlozky(i),
                    reakPr.getSmes().getNazevSlozky(i),c0[i],c[i],
                    xHmPoc[i],smReak.getxHmot(i),xMolPoc[i],smReak.getxMol(i));
        }
        double pomm = objemReaktoru/objemPrutok;
        if(smReak.sTbv(p)-TABS<reakPr.tVystup) report += "*** Došlo k odpaření kapalné reakční směsi! Havarijní stav.\n" ;
        report += String.format("Doba zdržení(s, min)= %6.1f  %4.2f%n",pomm,pomm/60);
        s = 0;
        for (i = 0; i < nReakci;i++) {
            s += xx[i]*c0[iRef[i]]*listReakci.get(i).getdH()*objemPrutok;
        }
        pomm = reakPr.hmotPrutok *(reakPr.tVystup * cOut - reakPr.tVstup * cIn)/1000;  // teplo akumulované na výstupu (kW)
        report += String.format("Uvolněné/spotřebované teplo(kW)= %7.5g%n",s/1000);
        report += String.format("Vyměněné teplo(kW)= %7.5g%n",(koefProstupu>0)? -s/1000-pomm:0);
        reakPr.spocten = true;
        reakPr.zUzlu = this;
        reakPr.xHmot = reakPr.sm.getxHmot();        
        reakPr.xMol = reakPr.sm.getxMol();
        reakPr.nazProudu = "Výstup z " + nazevAparatu;        
        reakPr.setVysledek(report);
        proudyOut.clear();
        proudyOut.add(reakPr);
        if(zobrazujVysledky) {
            new JDialogProud(null, nazevAparatu + "  výstupní proud (CSTR Verze " 
                        + VERZE + ")\n", reakPr); // modálně vyskakovací okno s výsledky              
        }
    }
    
/**
 * Metoda vyčíslí průběh reakce v reaktoru a v reakčním (výstupním) proudu<br>
 * s volbou zda zobrazit výsledky,
 * poskytne zreaagovanou směs.
 * @param tisk boolean
 * @throws fydat.NumerException
 */    
    public void reakce(boolean tisk) throws NumerException {
        if(tisk) zobrazujVysledky = true;
        reakce();
        zobrazujVysledky = tisk;
    }    
    
/**
 * Pravé strany pro Newtonovu metodu řešení reaktoru
 * @param xx vektor neznámých ref. konverzí a teploty double[]
 * @param ff vektor pravých stran double[]
 */    
  public void fun(double[] xx, double[] ff) {
      double sum, sumQ;       // sumář reakcí
      double s = 0;
      double x[] = new double[nSlozReak];
      
      x = xHmPoc.clone();   // počáteční hmotové složení
      if (rezim >0) {
        t = xx[nReakci]* reakPr.gettVstup() + TABS;      // abs. teplota v reaktoru
      } else {
        t =  reakPr.gettVstup() + TABS;     // izotermní
      }
      cOut = smReak.sCpW(t, p);      // specifické teplo výstupní směsi
      for (int i = 0; i < nSlozReagujicich; i++) {         
//      for (int i = 0; i < nSlozReak; i++) {         
          c[iPor[i]] = c0[iPor[i]];
          for (int j = 0; j < nReakci; j++) {
           // dopočet koncentrací z konverze   
           c[iPor[i]] += xx[j]*c0[iRef[j]]*(double)stechKoef(iPor[i], j);
           c[iPor[i]] *= hust/hust0;       // korekce koncentrace na hustotu
          }
          if (c[iPor[i]] < 0) c[iPor[i]] = 0;
          // dopočet hmot. zlomků reagujících složek
          x[iPor[i]] = c[iPor[i]] * smReak.getMolHmotSlozky(iPor[i])/hust;
          s += x[iPor[i]]; 
      }
      for (int i = 0;i < nSlozReagujicich;i++){
          x[iPor[i]] = x[iPor[i]]/s*(1-sumKonst);   // normalizace vzhledem k inertům
      }
        try {
            smReak.zadatSloz(true, x);    // aktualizace složení reakční směsi
        } catch (IOException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);
        }
      hust = smReak.sHust(t, p);    // aktualizovaá hustota reakční směsi 
      double[] reak = new double[listReakci.size()];
            for (int j = 0; j < listReakci.size();j++) {
                if(itypRovn == 0){
                  reak[j] = rychlostReakce(j, c);    // rychlost j-té reakce (mol/s/m3)
                } else {
                  reak[j] = rychlostReakce(j, c, cRovn);    // rychlost j-té rovn.reakce (mol/s/m3)    
                }
            }
      sum = 0;
      sumQ = 0;        
          for (int j = 0; j < listReakci.size();j++) {
              sum = sum + (double)stechKoef(iRef[j], j)* reak[j];
              sumQ += (double)stechKoef(iRef[j], j)* reak[j]*listReakci.get(j).getdH();
//              System.out.println("  fun j, stech, sum: "+iRef
//                      +"  "+stechKoef(iRef, j)+"  "+sum+"  reak="+reak[j]);
          }
      for (int j = 0; j < nReakci;j++) {
          ff[j] = c0[iRef[j]] - c[iRef[j]] + vf * sum;      // reziduum bilance ref složky j-té složky
//          System.out.println("ff i="+j+"   " +ff[j]+"   x="+xx[j]+"  sum="+sum);
      }
      if (rezim > 0) {  // neizotermní režim
          double pom = koefProstupu*teplosmenPlocha/reakPr.getHmotPrutok();
          if (rezim == 1) pom = 0;      // adiabatický režim
          ff[nReakci] = reakPr.gettVstup()*(cIn+pom)-xx[nReakci]*reakPr.gettVstup()*(cOut+pom)
                  +objemReaktoru/reakPr.getHmotPrutok()*sumQ;
//          System.out.println("ff i="+nReakci+"   " +ff[nReakci]+"   x="+xx[nReakci]+"  sumQ="+sumQ);
      }
  }
/**
 * Vrací rychlost j-té reakce - mocninová kinetika (mol/s/m3) nevratné reakce
 * @param jReak pořadové číslo reakce
 * @param c  koncentrace vystupující v reakční rychlosti
 * @return hodnota reakční rychlosti r=k(T).cA^nA.cB^nB.....
 */  
  private double rychlostReakce(int jReak, double[] c) {
      double rychlost;      // rychlost reakce (mol/s/m3)
      Reakce reak;
      reak = listReakci.get(jReak);     // j-tá reakce
      rychlost = reak.rychlKonst(t);    // rychlostní konstanta
      for (int m = 0; m < reak.getiKinSloz().length; m++) {  // přes kinet. složky reakce
          int por = porCisloLatky(reak.getiKinSloz()[m]);
          if (por < 0) {
              JOptionPane.showMessageDialog(null,
                        "Látka č. " + reak.getiKinSloz()[m] + 
                        "\nnení v reakční směsi " + reakPr.getSmes().getNazev() +". Končím!",
                         "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);
              return 0;
          }
          double cc; 
//          cc = cKonc(por);    // dopočet koncetrace z hmotového složení
          cc = c[por];        // koncentrace z iterací přes iPor
          cc = (cc > 0) ? cc: 0;
          if (cc < 0) {
              rychlost = 0;
          }
          rychlost *= Math.pow(cc, reak.getRadSlozky()[m]);
      }
      return rychlost;
  }
  
/**
 * Vrací rychlost j-té reakce - mocninová kinetika (mol/s/m3) rovnovážné reakce
 * @param jReak pořadové číslo reakce
 * @param c  koncentrace vystupující v reakční rychlosti
 * @param cRovn  rovnovážné koncentrace vystupující v reakční rychlosti
 * @return hodnota reakční rychlosti r=k(T).cA^nA.cB^nB.....
 */  
  private double rychlostReakce(int jReak, double[] c, double[] cRovn) {
      double rychlost;      // rychlost reakce (mol/s/m3)
      Reakce reak;
      reak = listReakci.get(jReak);     // j-tá reakce
      rychlost = reak.rychlKonst(t);    // rychlostní konstanta
      for (int m = 0; m < reak.getiKinSloz().length; m++) {  // přes kinet. složky reakce
          int por = porCisloLatky(reak.getiKinSloz()[m]);
          if (por < 0) {
              JOptionPane.showMessageDialog(null,
                        "Látka č. " + reak.getiKinSloz()[m] + 
                        "\nnení v reakční směsi " + reakPr.getSmes().getNazev() +". Končím!",
                         "CSTR Reaktor - reakce", JOptionPane.WARNING_MESSAGE);
              return 0;
          }
          double cc; 
//          cc = cKonc(por);    // dopočet koncetrace z hmotového složení
          cc = c[por] - cRovn[por];        // koncentrace z iterací přes iPor
          cc = (cc > 0) ? cc: 0;
          if (cc < 0) {
              rychlost = 0;
          }
          rychlost *= Math.pow(cc, reak.getRadSlozky()[m]);
      }
      return rychlost;
  }
  
/**
 * Vrací pořadové číslo látky cLatky ve směsi (kolikátá složka)
 * @param cLatky číslo látky z báze
 * @return pořadové číslo ve směsi
 */  
  private int porCisloLatky(int cLatky) {
      int por = -9999;
      for (int i = 0; i < reakPr.getSmes().getnSloz(); i++) {
          if (cLatky == reakPr.getSmes().getCisloSlozky(i)) por = i;
      }
      return por;
  }
/**
 * Vrací stechiometrický koeficient i-té složky reakční směsi
 * v j-té reakci
 * @param iSl pořadové číslo složky ve směsi
 * @param jReak pořadové číslo reakce
 * @return stechiometrický koeficient (int)
 */
  private int stechKoef(int iSl, int jReak) {
      int stech = 0;
      int cSloz = reakPr.getSmes().getCisloSlozky(iSl);
      Reakce r = listReakci.get(jReak);
      for (int i = 0; i < r.getiStech().length; i++) {
//          System.out.println("** cSloz, Stechi: "+cSloz+"  "+r.getiSlozky()[i]);
          if (cSloz == Math.abs(r.getiSlozky()[i])) {stech = r.getiStech()[i]; break; }
      }
      return stech;
  }
/**
 * Normalizuje hmotové zlomky reakční směsi a dosadí je do objektu směsi
 * @param x složení reakční směsi
 * @throws IOException 
 */  
  private void normalizujReagujici(double[] x) throws IOException {
      int i;
      double s = 0;
      for (i = 0; i < nSlozReagujicich; i++) {s += x[i];}
      for (i = 0; i < nSlozReagujicich; i++) {x[i] = x[i] / s * (1 -sumKonst); }
      // dosadit do objektu směsi
      double[] xHm = new double[nSlozReak];
      for (i = 0; i < nSlozReak; i++) xHm[i] = smReak.getxHmot(i);  // kvůli hm. zlomkům inertů
      for (i = 0; i < nSlozReagujicich; i++)  xHm[iPor[i]] = x[i];  // přepsat reakčními složkami
      smReak.zadatSloz(true, xHm);
//      for (i = 0; i < nSlozReak; i++) System.out.println("* i,x: "+i+"  "+xHm[i]);
  }
 /**
  * Vrací koncentraci i-té složky směsi (mol/m3)
  * @param i pořadové číslo složky směsi
  * @return koncentrace
  */   
    public double cKonc(int i) {
        // lze počítat také takto:
        // return smReak.sHust(t, p) * smReak.getxMol(i)/smReak.sMolHmot();
        return smReak.sHust(t, p) * smReak.getxHmot(i)/smReak.getMolHmotSlozky(i);
    }
///**
// * Vrací hmotový zlomek i-té složky (1) vypočtený z koncentrace
// * @param i pořadové číslo složky směsi
// * @return hmotový zlomek
// */    
//    public double xHmotC(int i) {
//    return c[i]/smReak.sHust(t, p) * smReak.getMolHmotSlozky(i);
//}
/**
 * Vrací pořadový index složky pro pořadové číslo neznámé
 * @param ii pořadové číslo neznámé
 * @return index složky ve směsi
 */
    private int iPorInv(int ii) {
        int k = -9999;
        for (int i = 0; i < iPor.length; i++) {
            if (iPor[i] == ii) k = i;
        }
        return k;
    }
/**
 * Vrací objem reaktoru (m3)
 * @return objem (double)
 */
    public double getObjemReaktoru() {
        return objemReaktoru;
    }
/**
 * Nastavuje objem reaktoru (m3)
 * @param objemReaktoru 
 */
    public void setObjemReaktoru(double objemReaktoru) {
        this.objemReaktoru = objemReaktoru;
        myInit();
    }
/**
 * Vrací objemový průtok (m3/s)
 * @return průtok
 */
    public double getObjemPrutok() {
        return objemPrutok;
    }
///**
// * Nastavuje objemový průtok (m3/s)
// * @param objemPrutok 
// */
//    public void setObjemPrutok(double objemPrutok) {
//        this.objemPrutok = objemPrutok;
//    }
/**
 * Vrací reakční proud
 * @return proud
 */
    public Proud getReakPr() {
        return reakPr;
    }
/**
 * Nastavuje reakční proud
 * @param reakPr 
 */
    public void setReakPr(Proud reakPr) {
        this.reakPr = reakPr;
    }
/**
 * Vrací teplosměnnou plochu (m2)
 * @return plocha
 */
    public double getTeplosmenPlocha() {
        return teplosmenPlocha;
    }
/**
 * Nastavuje teplosměnnou plochu (m2)
 * @param teplosmenPlocha 
 */
    public void setTeplosmenPlocha(double teplosmenPlocha) {
        this.teplosmenPlocha = teplosmenPlocha;
    }
/**
 * Vrací koeficient prostupu tepla (W/m2/K)
 * @return koeficient
 */
    public double getKoefProstupu() {
        return koefProstupu;
    }
/**
 * Nastavuje koeficient prostupu tepla (J/m2/K)
 * @param koefProstupu 
 */
    public void setKoefProstupu(double koefProstupu) {
        this.koefProstupu = koefProstupu;
    }
/**
 * Vrací teplotu chlazení (°C)
 * @return teplota
 */
    public double getT0() {
        return t0;
    }
/**
 * Nastavuje teplotu chlazení (°C)
 * @param t0 (°C)
 */
    public void setT0(double t0) {
        this.t0 = t0;
    }
/**
 * Vrací tepelný režim reaktoru
 * 0-izotermní; 1-adiabatický; 2-výměna tepla
 * @return režim
 */
    public int getRezim() {
        return rezim;
    }
/**
 * Nastavuje tepelný režim reaktoru
 * 0-izotermní; 1-adiabatický; 2-výměna tepla
 * @param rezim 
 */
    public void setRezim(int rezim) {
        this.rezim = rezim;
    }
/**
 * Vrací, zda existuje zpětný chladič na reaktoru
 * @return boolean
 */
    public boolean isZpetnyChladic() {
        return zpetnyChladic;
    }
/**
 * Nastavuje existenci/neexistenci zpětného chladiče
 * na míchaném reaktoru
 * @param zpetnyChladic 
 */
    public void setZpetnyChladic(boolean zpetnyChladic) {
        this.zpetnyChladic = zpetnyChladic;
    }
  
 /**
  * Nastavuje parametry přestupu tepla
  * @param plocha (m2)
  * @param koef (W/m2/K)
  * @param tChlaz (°C)
  */
    public void paramPrestupuTepla(double plocha, double koef, double tChlaz) {
        teplosmenPlocha = plocha;
        koefProstupu = koef;
        t0 = tChlaz;
        rezim = 2;      // výměna tepla
        if (koefProstupu < 0.01) rezim = 1;     // adiabatický
        myInit();
    }
}
