/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import javax.swing.JOptionPane;

/**
 * Třída popisuje proudění elementem kolenem, především vyčísluje tlakovou ztrátu.
 * @author V.Hlubuček
 */
public class Koleno extends Element{

    private double prumerTrubky, drsnost, vyskovyRozdil;
    private double polomerKolena, uhelKolena;
/**
 * Konstruktor kolena
 * @param prumerTrubky průměr trubky tvořící koleno (m)
 * @param polomerKolena poloměr zahnutí kolena (°)
 * @param uhelKolena  úhel, který tvoří úseč kolena (°) (obvykle 90°)
 */
    public Koleno(Proud pr, double prumerTrubky, double polomerKolena, double uhelKolena) {
        super();
        this.pr = pr;
        nazElementu = "Koleno";        
        drsnost = 0.0001;
        this.prumerTrubky = prumerTrubky;
        this.polomerKolena = polomerKolena;
        this.uhelKolena = uhelKolena;
        myInit();        
    }

    public Koleno() {
        nazElementu = "Koleno";
        drsnost = 0.0001; 
        myInit();                
    }

/**
 * Inicializace elementu koleno
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[7];
        tValue = new String[7];
        lTexty[0] = "Koleno";
        lTexty[1] = "Průměr trubky(m)";
        lTexty[2] = "Drsnost";
        lTexty[3] = "Poloměr kolena (m)";
        lTexty[4] = "Úhel kolena(°)";
        lTexty[5] = "Výškový rozdíl(m)";
        lTexty[6] = "";
        tValue[1] = "0,05";
        tValue[2] = "0,0004";
        tValue[3] = "0,4";
        tValue[4] = "80";
        tValue[5] = "0";               
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        prumerTrubky = doubleFromString(tValue[1]);
        drsnost = doubleFromString(tValue[2]);
        polomerKolena = doubleFromString(tValue[3]);
        uhelKolena = doubleFromString(tValue[4]);
        vyskovyRozdil = doubleFromString(tValue[5]);
    }    
    
    
/**
 * Počítá tlakovou ztrátu (pa) na koleni
 * @param pr proud
 * @return tlaková ztráta (Pa)
 */    
    @Override
    public double tlakZtrata() {
        boolean kapalina;
        double d, gsi, ro, t, p, v;
        d = prumerTrubky;
//        tVstup = pr.gettVstup();
        t = tVstup+TABS;    // absolutní teplota (K)
//        pVstup = pr.getpVstup();        
        p = pVstup;
        if (pVstup*tVstup <= 0 && pr.isTisk()) {
           JOptionPane.showMessageDialog(null,
              "V elementu " + nazElementu + " pr. (" + pr.cProudu + ") nulový tlak nebo teplota !",
              "Koleno", JOptionPane.WARNING_MESSAGE);            
            return -9999;   // chyba
        }
        kapalina = false;
        if (t < pr.getSmes().sTbv(p)) kapalina = true;  
        if (kapalina) {
            ro = pr.getSmes().sHustL(t);
        } else {
            ro = pr.getSmes().srHustGW(t, p);
        }
        v = 4 * pr.getHmotPrutok() / ro / Math.PI / d / d;
        gsi = 0.131 + 0.16 * Math.pow(d / prumerTrubky, 3.5);
        pVystup = p - 9.81 * ro * vyskovyRozdil -  gsi * v * v * ro / 2;
        tVystup = tVstup;
        rychlVstup = v;
        rychlVystup = v;
        report = "Koleno (" + nazElementu + ") (l)\n";
        report += " průměr potrubí    (m) " + String.format("%5.2f%n",prumerTrubky);
        report += " poloměr oblouku   (m) " + String.format("%5.2f%n",polomerKolena);
        report += " úhel ohybu        (°) " + String.format("%5.2f%n",uhelKolena);
        report += " výškový spád      (m) "  + String.format("%5.2f%n",vyskovyRozdil);      
        report += " drsnost potrubí   (m) " + String.format("%7.5f%n",drsnost);
        report +=  " hustota       (kg/m3) " + String.format("%5.2f%n",ro);
        report +=  " teplota bodu varu(°C) " + String.format("%5.1f%n",pr.getSmes().sTbv(p)-TABS);
        report +=  " viskozita      (Pa.s) " + String.format("%6.4g%n",pr.getSmes().sViskL(t));
        report +=  " rychlost        (m/s) " + String.format("%6.3f%n",v);
//        report +=  " Reynoldsovo č.  (1) " + String.format("%6.4g%n",nRe);
        report +=  " součinitel odporu (1) " + String.format("%6.4f%n",gsi);
        report +=  " teplota          (°C) " + String.format("%5.1f%n",tVstup);
        report +=  " tlak na vstupu   (Pa) " + String.format("%7.0f%n",pVstup);
        report +=  " tlak na výstupu  (Pa) " + String.format("%7.0f%n",pVystup);
        report +=  " tlaková ztráta   (Pa) " + String.format("%6.1f%n",p - pVystup);
        return pVstup - pVystup;   // tlaková ztráta
  }

    @Override
    public String vysledkyElement() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
/**
 * Vrací průměr trubky kolena
 * @return průměr trubky
 */
    public double getPrumerTrubky() {
        return prumerTrubky;
    }
/**
 * Nastavuje průměr trubky kolena
 * @param prumerTrubky 
 */
    public void setPrumerTrubky(double prumerTrubky) {
        this.prumerTrubky = prumerTrubky;
    }
/**
 * Vrací drsnost povrchu (m)
 * @return drsnost
 */
    public double getDrsnost() {
        return drsnost;
    }
/**
 * Nastavuje drsnost povrchu (m)
 * @param drsnost 
 */
    public void setDrsnost(double drsnost) {
        this.drsnost = drsnost;
    }
/**
 * Vrací výškový rozdíl (m)
 * @return výškový rozdíl
 */
    public double getVyskovyRozdil() {
        return vyskovyRozdil;
    }
/**
 * Nastavuje výškový rozdíl (m)
 * @param vyskovyRozdil 
 */
    public void setVyskovyRozdil(double vyskovyRozdil) {
        this.vyskovyRozdil = vyskovyRozdil;
    }
/**
 * Vrací poloměr záhyby kolena (m)
 * @return poloměr
 */
    public double getPolomerKolena() {
        return polomerKolena;
    }
/**
 * Nastavuje poloměr záhyby kolena (m)
 * @param polomerKolena 
 */
    public void setPolomerKolena(double polomerKolena) {
        this.polomerKolena = polomerKolena;
    }
/**
 * Vrací úhel kolena (°)
 * @return úhel
 */
    public double getUhelKolena() {
        return uhelKolena;
    }
/**
 * Nastavuje úhel kolena (°)
 * @param uhelKolena 
 */
    public void setUhelKolena(double uhelKolena) {
        this.uhelKolena = uhelKolena;
    }
    
}
