/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import javax.swing.JOptionPane;

/**
 * Třída popisuje tlakovou ztrátu v elementu pozvolné zúžení nebo rozšíření
 * @author V. Hlubuček
 */
public class KonickaTrubka extends Element {

    private double prumerVstup,  drsnost, vyskovyRozdil;
    private double prumerVystup, uhel;   // kladný úhel rozšíření, záporný zúžení 
/**
 * Konstruktor konické trubky
 * @param prumerVstup vstupní průměr (m)
 * @param prumerVystup výstupní průměr (m)
 * @param uhel úhel rozevření/sevření
 */
    public KonickaTrubka(Proud pr, double prumerVstup, double prumerVystup, double uhel) {
        super();
        this.pr = pr;
        nazElementu = "";        
        drsnost = 0.0001;        
        this.prumerVstup = prumerVstup;
        this.prumerVystup = prumerVystup;
        this.uhel = uhel;
        myInit();
    } 
    
/**
 * Inicializace elementu konická trubka
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[7];
        tValue = new String[7];
        lTexty[0] = "Konická trubka";
        lTexty[1] = "Průměr vstupní(m)";
        lTexty[2] = "Průměr vstupní(m)";
        lTexty[3] = "Drsnost";
        lTexty[4] = "úhel(°)";
        lTexty[5] = "Výškový rozdíl(m)";
        lTexty[6] = "";
        tValue[1] = "0,03";
        tValue[2] = "0,05";
        tValue[3] = "0,0005";
        tValue[4] = "15";
        tValue[5] = "0";
        
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        prumerVstup = doubleFromString(tValue[1]);
        prumerVystup = doubleFromString(tValue[2]);
        drsnost = doubleFromString(tValue[3]);
        uhel = doubleFromString(tValue[4]);
        vyskovyRozdil = doubleFromString(tValue[5]);
    }
    
    
/**
 * Počítá tlakovou ztrátu (Pa) na konické trubce
 * @param pr proud
 * @return tlaková ztráta (Pa)
 */    
    @Override
    public double tlakZtrata() {
        boolean kapalina;
        double din, dout, gsi = 0, ro, ro1, ro2, t, p, pp, p2, pom, vin, vout = 0, visk = 0, visk2 = 0;
        double delta, nRe1, nRe2, h, h1, h2;
        din = prumerVstup;
        dout = prumerVystup;
        delta = uhel*Math.PI/180;
        t = tVstup+TABS;    // absolutní teplota (K)       
        p = pVstup;
        if (pVstup*tVstup <= 0 && pr.isTisk()) {
           JOptionPane.showMessageDialog(null,
              "V elementu " + nazElementu + " pr. (" + pr.cProudu + ") nulový tlak nebo teplota !",
              "Konická trubka", JOptionPane.WARNING_MESSAGE);            
            return -9999;   // chyba
        }
        if (din > dout && uhel > 0) {
           JOptionPane.showMessageDialog(null,
              "V elementu " + nazElementu + " pr. (" + pr.cProudu + ") upravuji úhel na záporný !",
              "Zúžení", JOptionPane.WARNING_MESSAGE);            
            uhel = -uhel;   
        }        
        kapalina = false;
        if (t < pr.getSmes().sTbv(p)) kapalina = true;
        pom = dout*dout/din/din;
        if (kapalina) {
            ro = pr.getSmes().sHustL(t);
            visk = pr.getSmes().sViskL(t);      
            vin = 4 * pr.getHmotPrutok() / ro / Math.PI / din / din;
            vout = vin/pom;
            nRe1 = vin * din * ro / visk;
            nRe2 = nRe1 / pom;
            h1 = hltr(nRe1, din, drsnost);
            h2 = hltr(nRe2, dout, drsnost);
            h = (h1 + h2) / 2;
            if (din < dout) {  // rozšiřující se
              gsi = (vin - vout)*(vin - vout) / vout / vout * Math.sin(delta) + h / 8 / Math.sin(delta / 2) * (pom * pom - 1);
              if (nazElementu.equals("")) nazElementu = "Rozšířující se trubka";
            } else {            // zužující se
              gsi = h / 8 / Math.sin(delta / 2) * (1 - pom * pom);
              if (nazElementu.equals("")) nazElementu = "Zužující se trubka";          
            }
            pVystup = p - 9.81 * ro * vyskovyRozdil -  gsi * vout * vout * ro / 2;
            tVystup = tVstup;
            rychlVystup = vout;
            rychlVstup = vin;
            report = "Konická trubka (" + nazElementu + ") (l)\n";
            report += " průměr vstupní    (m) " + String.format("%5.2f%n",prumerVstup);
            report += " průměr výstupní   (m) " + String.format("%5.2f%n",prumerVystup);
            report += " úhel konicity     (°) " + String.format("%5.2f%n",uhel);
            report += " výškový spád      (m) "  + String.format("%5.2f%n",vyskovyRozdil);      
            report += " drsnost potrubí   (m) " + String.format("%7.5f%n",drsnost);
            report +=  " hustota       (kg/m3) " + String.format("%5.2f%n",ro);
            report +=  " teplota bodu varu(°C) " + String.format("%5.1f%n",pr.getSmes().sTbv(p)-TABS);
            report +=  " viskozita      (Pa.s) " + String.format("%6.4g%n",pr.getSmes().sViskL(t));
            report +=  " rychlost vstup  (m/s) " + String.format("%6.3f%n",vin);
            report +=  " rychlost výstup (m/s) " + String.format("%6.3f%n",vout);
    //        report +=  " Reynoldsovo č.  (1) " + String.format("%6.4g%n",nRe);
            report +=  " součinitel odporu (1) " + String.format("%6.4f%n",gsi);
            report +=  " teplota          (°C) " + String.format("%5.1f%n",tVstup);
            report +=  " tlak na vstupu   (Pa) " + String.format("%7.0f%n",pVstup);
            report +=  " tlak na výstupu  (Pa) " + String.format("%7.0f%n",pVystup);
            report +=  " tlaková ztráta   (Pa) " + String.format("%6.1f%n",p - pVystup);        
        }
        if (!kapalina) {
            ro1 = pr.getSmes().srHustGW(t, p);
            visk = pr.getSmes().srViskG(t, p);
            vin = 4 * pr.getHmotPrutok() / ro1 / Math.PI / din / din;
            nRe1 = vin * din * ro1 / visk;
            h1 = hltr(nRe1, din, drsnost);
            p2 = p;
            pp = p + 20;
            while (Math.abs(pp - p2)<1e-2) {
                p2 = pp;
                ro2 = pr.getSmes().srHustGW(t, p2);
                visk2 = pr.getSmes().srViskG(t, p2);
                vout = 4 * pr.getHmotPrutok() / ro2 / Math.PI / dout / dout;
                nRe2 = vout * dout * ro2 / visk2;
                h2 = hltr(nRe2, dout, drsnost);
                h = (h1 + h2) / 2;
                gsi = (vin - vout)*(vin - vout) / vout / vout * Math.sin(delta) + h / 8 / Math.sin(delta / 2) * (pom * pom - 1);
                ro = 0.5 * (ro1 + ro2);
                pp = p - 9.81 * ro * vyskovyRozdil -  gsi * vout * vout * ro2 / 2;
            }
            pVystup = pp;
            report = "Konická trubka (" + nazElementu + ") (g)\n";
            report += " průměr vstupní    (m) " + String.format("%5.2f%n",prumerVstup);
            report += " průměr výstupní   (m) " + String.format("%5.2f%n",prumerVystup);
            report += " úhel konicity     (°) " + String.format("%5.2f%n",uhel);
            report += " výškový spád      (m) "  + String.format("%5.2f%n",vyskovyRozdil);      
            report += " drsnost potrubí   (m) " + String.format("%7.5f%n",drsnost);
            report +=  " hustota vstup (kg/m3) " + String.format("%5.3f%n",ro1);
            report +=  " hustota výstup(kg/m3) " + String.format("%5.3f%n",ro1);
            report +=  " teplota bodu varu(°C) " + String.format("%5.1f%n",pr.getSmes().sTbv(p)-TABS);
            report +=  " viskozita      (Pa.s) " + String.format("%6.4g%n",visk);
            report +=  " rychlost vstup  (m/s) " + String.format("%6.3f%n",vin);
            report +=  " rychlost výstup (m/s) " + String.format("%6.3f%n",vout);
            report +=  " Reynoldsovo č.    (1) " + String.format("%6.4g%n",nRe1);
            report +=  " součinitel odporu (1) " + String.format("%6.4f%n",gsi);
            report +=  " teplota          (°C) " + String.format("%5.1f%n",tVstup);
            report +=  " tlak na vstupu   (Pa) " + String.format("%7.0f%n",pVstup);
            report +=  " tlak na výstupu  (Pa) " + String.format("%7.0f%n",pVystup);
            report +=  " tlaková ztráta   (Pa) " + String.format("%6.1f%n",p - pVystup);
        }
        return pVstup - pVystup;   // tlaková ztráta
    }

    @Override
    public String vysledkyElement() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
/**
 * Vrací vstupní průměr konické trubky (m)
 * @return vstupní průměr
 */
    public double getPrumerVstup() {
        return prumerVstup;
    }
/**
 * Nastavuje vstupní průměr konické trubky (m)
 * @param prumerVstup 
 */
    public void setPrumerVstup(double prumerVstup) {
        this.prumerVstup = prumerVstup;
    }
/**
 * Vrací abs. drsnost potrubí (m)
 * @return drsnost
 */
    public double getDrsnost() {
        return drsnost;
    }
/**
 * Nastavuje abs. drsnost potrubí (m)
 * @param drsnost 
 */
    public void setDrsnost(double drsnost) {
        this.drsnost = drsnost;
    }
/**
 * Vrací výškový rozdíl (m)
 * @return 
 */
    public double getVyskovyRozdil() {
        return vyskovyRozdil;
    }
/**
 * Nastavuje výškový rozdíl (m)
 * @param vyskovyRozdil 
 */
    public void setVyskovyRozdil(double vyskovyRozdil) {
        this.vyskovyRozdil = vyskovyRozdil;
    }
/**
 * Vrací výstupní průměr konické trubky (m)
 * @return průměr
 */
    public double getPrumerVystup() {
        return prumerVystup;
    }
/**
 * Nastavuje výstupní průměr konické trubky (m)
 * @param prumerVystup 
 */
    public void setPrumerVystup(double prumerVystup) {
        this.prumerVystup = prumerVystup;
    }
/**
 * Vrací úhel rozšíření/zúžení konické trubky (°)
 * @return úhel
 */
    public double getUhel() {
        return uhel;
    }
/**
 * Nastavuje úhel rozšíření/zúžení konické trubky (°)
 * @param uhel 
 */
    public void setUhel(double uhel) {
        this.uhel = uhel;
    }
}
