/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import fydatlib.Smes;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import org.jdom2.JDOMException;

/**
 * Třída popisuje technologický proud, obsahuje jeho složky, složení,<br>
 * hmotový průtok, teplotu a tlak.<br> Tuto třídu používají všechny 
 * uživatelské chemicko-inženýrské programy.
 * @author V. Hlubuček
 */
public class Proud {
    
    public final double TABS = 273.15;
    protected int cProudu;                // číslo proudu, výchozí uzel, následující uzel
    protected int nElem = 0;              // počet elementů proudu
    protected int tag = 0;                // pomocná proměnná
    protected boolean spocten;            // proud byl spočten
    protected boolean tisk;               // dělat reporty?
    protected boolean bylTisk;            // pomocná interní proměnná
    protected String VERZE = "V1.3";	  // verze třídy
    protected String nazProudu;		  // název proudu
    protected String vysledek;            // textový výsledek proudu
    protected double hmotPrutok;		// hmotový průtok (kg/s)
    protected double tVstup, tVystup, pVstup, pVystup;	// vstupní a výstupní teploty(°C) a tlaky(?)
    protected double tlakZtrata;          // tlaková ztráta (během výpočtu se průběžně mění)
    protected double dMin;                // minimální průměr
    protected double pom;                 // pomocná proměnná (batch aparáty)
    protected Smes sm;			  // směs látek
    protected Aparat zUzlu, doUzlu;       // výchozí uzel, následující uzel (aparát)
    protected double[] xHmot, xMol;	  // hmotové a molární zlomky složek směsi
    protected List<Element>  prvky = new ArrayList();    // kolekce trubek proudu

    public Proud() {
    }
/**
 * Konstruktor proudu
 * @param cProudu číslo proudu (int)
 * @param nazProudu textový název proudu
 */    
    public Proud(int cProudu, String nazProudu) {
        this.cProudu = cProudu;
        this.nazProudu = nazProudu;
    }
/**
 * Konstruktor proudu
 * @param cProudu číslo proudu (int)
 * @param nazProudu textový název proudu
 * @param hmotPrutok hmotový průtok proudem(kg/s)
 */    
    public Proud(int cProudu, String nazProudu, double hmotPrutok) {
        this.cProudu = cProudu;
        this.nazProudu = nazProudu;
        this.hmotPrutok = hmotPrutok;
    }
/**
 * Konstruktor proudu
 * @param cProudu číslo proudu (int)
 * @param zUzlu číslo uzlu, z kterého proud vychází
 * @param doUzlu číslo uzlu do kterému proud směřuje
 * @param nazProudu textový název proudu
 */    
    public Proud(int cProudu, Aparat zUzlu, Aparat doUzlu, String nazProudu) {
        this.cProudu = cProudu;
        this.zUzlu = zUzlu;
        this.doUzlu = doUzlu;
        this.nazProudu = nazProudu;
    }
    
    public Proud(Proud pr) {
        // případně    BeanUtils.copyProperties() 

        try {
            this.cProudu = pr.cProudu;
            this.zUzlu = pr.zUzlu;
            this.doUzlu = pr.doUzlu;
            this.nazProudu = pr.nazProudu;
            this.hmotPrutok = pr.hmotPrutok;
            this.tVstup = pr.tVstup;
            this.pVstup = pr.pVstup;
            this.tVystup = pr.tVstup;
            this.pVystup = pr.pVstup;            
            this.sm = pr.getSmes().kopirujSmes(pr.getSmes());
        } catch (JDOMException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);                 
        } catch (IOException ex) {
            Logger.getLogger(Proud.class.getName()).log(Level.SEVERE, null, ex);
        } catch (NullPointerException ex) {
            Logger.getLogger(Proud.class.getName()).log(Level.SEVERE, null, ex);            
        }      
    }
    
/**
 * Konstruktor proudu - chladicí voda
 * @param cProudu číslo proudu (int)
 * @param teplota (teplota chladicí vody)
 * @param hmotPrutok hmotový průtok proudem(kg/s)
 */    
    public Proud(int cProudu, double teplota , double hmotPrutok) {
        this.cProudu = cProudu;
        this.nazProudu = "Chladicí voda";
        this.hmotPrutok = hmotPrutok;
        tVstup = teplota;
        pVstup = 300000;
        try {
            double[] x = new double[1];
            x[0] = 1;
            sm = new Smes("Chladicí voda", 1);
            sm.zadatSloz(true,x);
        } catch (JDOMException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);                 
        } catch (IOException ex) {
            Logger.getLogger(Proud.class.getName()).log(Level.SEVERE, null, ex);
        }
    }    
    
/**
 * Metoda slouží k hromadnému zadání hlavních parametrů proudu
 * @param cProudu číslo proudu (int)
 * @param nazProudu  textový název proudu
 * @param zUzlu číslo uzlu, z kterého proud vychází
 * @param doUzlu číslo uzlu do kterému proud směřuje
 * @param hmotPrutok hmotový průtok proudem(kg/s)
 * @param tVstup vstupní teplota proudu(°C)
 * @param pVstup vstupní tlak proudu(Pa)
 */    
    public void zadejParametryProudu(int cProudu, String nazProudu, Aparat zUzlu, Aparat doUzlu,
            double hmotPrutok,double tVstup, double pVstup) {
        this.cProudu = cProudu;
        this.nazProudu = nazProudu;
        this.zUzlu = zUzlu;
        this.doUzlu = doUzlu;
        this.hmotPrutok = hmotPrutok;
        this.tVstup = tVstup;
        this.pVstup = pVstup;
        this.tVystup = tVstup;
        this.pVystup = pVstup;
    }
/**
 * Interaktivní zadání parametrů proudu
 */    
    public void zadejParametryInteraktivne()  {
        JDialogProudInp jdpi;
        if(hmotPrutok <= 0) {
            jdpi = new JDialogProudInp(null, true); // interaktivní vstup dat proudu
        } else {
            jdpi = new JDialogProudInp(null, this,true);
        }
        Proud pr = (jdpi.getProud());
        this.cProudu = pr.cProudu;
        this.zUzlu = pr.zUzlu;
        this.doUzlu = pr.doUzlu;
        this.nazProudu = pr.nazProudu;
        this.hmotPrutok = pr.hmotPrutok;
        this.tVstup = pr.tVstup;
        this.pVstup = pr.pVstup;
        this.tVystup = pr.tVstup;
        this.pVystup = pr.pVstup; 
        this.sm = pr.sm;
        xHmot = sm.getxHmot();
        xMol = sm.getxMol();
    }
/**
 * Dosadí stejnou teplotu a tlak na výstup, jako je na vstupu
 */    
    public void vystupJakoVstup() {
        tVystup = tVstup;
        pVystup = pVstup;
    }
/**
 * Přidání elementu do seznamu proudu
 * @param el element
 */
    public void pridatElement(Element el) {
        prvky.add(el);  
        nElem++;
    }
/**
 * Přidání elementu do seznamu proudu na pozici i
 * @param el element
 * @param i index
 */    
    public void pridatElement(Element el, int i) {
        prvky.add(i,el);
        nElem++;
    }    
/**
 * Odebrání elementu ze seznamu proudu
 * @param el 
 */ 
    public void odebratElement(Element el) {
        prvky.remove(el);
        nElem--;
    } 
/**
 * Generalizovaná metoda výpočtu proudu
 */    
    public void spoctiAparat(){
        tlakZtrata();
    }    
/**
 * Počítá tlakovou ztrátu (Pa) na proudu pro zadaný vstupní tlak(Pa)
 * a hmotový průtok (kg/s)
 * @return tlaková ztráta
 */    
    public double tlakZtrata() {
        bylTisk = tisk;     // zaznamenat stav požadavku na tisk reportů
        if (Math.abs(hmotPrutok) < 1e-6) {
            JOptionPane.showMessageDialog(null,
                "Zadán nulový hmotový průtok u proudu " + cProudu + " Končím!",
                 "Proud - tlakZtrata", JOptionPane.WARNING_MESSAGE);
            return -9999;
    }                 
        vysledek = String.format("*** Celkové výsledky z proudu: %s %nZ uzlu %s do uzlu  %s%n",
                nazProudu, zUzlu,doUzlu);
        vysledek += String.format("Hmotový průtok (kg/s): %6.5g%n",hmotPrutok);
        double s = 0, p = 0, t = 0;
        for (Element el:prvky) {
            p = pVstup - s;     // vstupní tlak na elementu
            t = tVstup;            
            el.pVstup = p;
            el.tVstup = t;
            s += el.tlakZtrata();
            p = el.pVystup;     // výstupní tlak na elementu
            t = el.tVystup;
            vysledek = vysledek + el.report;    // přidání výsledku z reportu elementu
        }
        pVystup = p;
        tVystup = t;
        vysledek += String.format("%nVýstupní tlak z proudu (Pa) a teplota (°C) proudu: %6.0f   %4.0f%n",
              pVystup, tVystup);
        vysledek += String.format("Celková tlaková ztráta (Pa): %6.0f", s);
        vysledek += String.format("   t.j. %5.1f %s ", s/pVstup*100,"%"); 
        if (tisk) System.out.println(vysledek);     // výsledky na systémovou konzoli
        tisk = bylTisk;     // zpětné nastavení původního stavu tisku        
        return s;
    }
/**
 * Dopočítá hmotový průtok (kg/s) proudu pro zadané tlaky (Pa) na vstupu a výstupu proudu.
 * V tom případě ponechte odhad hmotového průtoku raději nulový
 * a metoda si ho odhadne sama.
 * @return průtok
 */
    public double dopoctiPrutok() {
        int iter = 0, itmax = 50;       // iterační proměnná
        bylTisk = tisk;     // zaznamenat stav požadavku n atisk reportů
        tisk = false;
        double m, m1 = 0, dP, dP1, dP2;
        System.out.println(" * P vstup, výstup: "+pVstup+"  "+pVystup);
        dMin = dMin();
        dP = pVstup - pVystup;
        m = hmotPrutok; 
        // počáteční odhad hmotového průtoku
        if (Math.abs(hmotPrutok) < 1e-6) {
            double ro = getSmes().sHust(tVstup+TABS, pVstup);
            if (ro > 400) {
                m = 0.3 * ro * dMin * dMin;     // pro kapalinu            
            } else {
                m = 1 * ro * dMin * dMin;     // pro kapalinu                            
            }
            if (m < 0) { m = 250 * dMin * dMin; }   // při dvoufázovém toku
            hmotPrutok = m;
        }
        do { 
           hmotPrutok += 0.005; 
           dP2 = tlakZtrata();
           hmotPrutok -= 0.005; 
           dP1 = tlakZtrata();           
           m1 = m - 0.7* (dP1 - dP)/(dP2 - dP1)*0.005;
           if (m1 < 0) m1 = 0.01;
           hmotPrutok = m1;
           m = m1;
            System.out.println("m, dP, dP1: "+m+" "+dP+"  "+dP1 );
            iter++;
            if (iter == itmax) {
                JOptionPane.showMessageDialog(null,
                    "Překročen max. počet iterací " + itmax + " !",
                     "Proud - dopoctiPrutok", JOptionPane.WARNING_MESSAGE);            
        }               
        } while (Math.abs(dP - dP1)> 1 && iter < itmax);
        hmotPrutok = m1;
        tisk = bylTisk;     // zpětné nastavení původního stavu tisku
        System.out.println(" ** P vstup, výstup: "+pVstup+"  "+pVystup+"   iter= "+iter);
        return m1;
    }
/**
 * Zjistí minimální průměr armatur v proudu
 * @return průměr
 */
    private double dMin() {
        double d = 999;
        for (Element el:prvky) {
            if (el instanceof Trubka) {
                d = Math.min(d, ((Trubka) el).getPrumerTrubky());
            }
            if (el instanceof KonickaTrubka) {
                d = Math.min(d, (Math.min(((KonickaTrubka) el).getPrumerVstup(),((KonickaTrubka) el).getPrumerVstup())));                
            }
        }
        return d;
    }
 /**
  * Pomocné zobrazení výsledků
  */   
    public void zobrazVysledek() {
        vysledek = "";
        pVystup = pVstup;
        for (Element el:prvky) {
            el.tlakZtrata();
            vysledek = vysledek + el.report;
            pVystup -= el.pVstup - el.pVystup;  // odečítání tlakové ztráty
        }
        System.out.println("*** Celkové výsledky z proudu: "+nazProudu+"\nZ uzlu "+zUzlu+" do uzlu "+doUzlu);
        System.out.println(vysledek); 
        System.out.println("Výstupní tlak proudu: "+String.format("%6.0f", pVystup));
    }
/**
 * Zkopíruje obsah proudu do nové proměnné
 * @param pr Proud 
 * @return Proud zkopírovaný proud
 */    
    public Proud kopirujProud(Proud pr)  {
        Proud prn = new Proud(pr.cProudu, pr.nazProudu, pr.hmotPrutok);
        Smes smn;
        if(pr.sm.getChyba() < 0) {
            int input = JOptionPane.showOptionDialog(null, "Směs není plně definována: \n" 
                    + pr.sm.getNazev() + ", chyba= " + pr.sm.getChyba() + "\nChcete ukončit program?", 
                    "Proud - kopíruj", JOptionPane.OK_CANCEL_OPTION, 
                    JOptionPane.INFORMATION_MESSAGE, null, null, null);
            // chyba = -2 Nebylo zadáno složení směsi
            if(input == JOptionPane.OK_OPTION) System.exit(0);
            return null;                       
        }
        try {
            smn = pr.sm.kopirujSmes(pr.sm);
            prn.setSmes(smn);
        } catch (JDOMException ex) {
            Logger.getLogger(CSTRReaktor.class.getName()).log(Level.SEVERE, null, ex);                 
        } catch (IOException ex) {
            Logger.getLogger(Proud.class.getName()).log(Level.SEVERE, null, ex);
        }
        prn.zUzlu = pr.zUzlu;
        prn.doUzlu = pr.doUzlu;
        prn.tVstup = pr.tVstup;
        prn.tVystup = pr.tVystup;
        prn.pVstup = pr.pVstup;
        prn.pVystup = pr.pVystup; 
        prn.prvky = pr.prvky;   // kolekce prvků
        prn.xMol = pr.xMol;
        prn.xHmot = pr.xHmot;
        prn.nElem = pr.nElem;
        prn.tlakZtrata = pr.tlakZtrata;
        prn.dMin = pr.dMin;
        prn.tisk = pr.tisk;
        prn.vysledek = pr.vysledek;
        prn.pom = pr.pom;
        prn.VERZE = pr.VERZE;
        return prn;
    }
/**
 * Metoda kontroluje zda byly zadány hlavní parametry proudu
 * umožňující jeho výpočet
 * @return boolean ano-ne
 */    
        public boolean jeProudZadan() {
            boolean neni = false;
            if(this == null) return false;
            if(hmotPrutok <= 0) neni = true;
            if(tVstup <= 0 ) neni = true;
            if(pVstup <= 0 ) neni = true;
            if(sm == null) neni = true;
            return (!neni);
        }
/**
 * Metoda kontroluje zda byly zadány hlavní parametry proudu
 * umožňující jeho výpočet a to včetně napojení aparátů
 * @param aparaty jakákoliv hodnota kontroluje i aparáty
 * @return boolean ano-ne
 */    
        public boolean jeProudZadan(boolean aparaty) {
            boolean neni = false;
            if(this == null) return false;            
            if(hmotPrutok <= 0) neni = true;
            if(tVstup <= 0 ) neni = true;
            if(pVstup <= 0 ) neni = true;
            if(sm == null) neni = true;
            if(zUzlu == null) neni = true;
            if(doUzlu == null) neni = true;
            return (!neni);
        }
    
// gettery a settery  ---------------------------------------------------    
/**
 * Vrací číslo proudu
 * @return číslo proudu (int)
 */    
    public int getcProudu() {
        return cProudu;
    }
/**
 * Nastavuje číslo proudu
 * @param cProudu 
 */
    public void setcProudu(int cProudu) {
        this.cProudu = cProudu;
    }
/**
 * Vrací výchozí aparát, odkud proud vychází
 * @return Aparat
 */
    public Aparat getzUzlu() {
        return zUzlu;
    }
/**
 * Nastavuje výchozí aparát, odkud proud vychází
 * @param zUzlu 
 */
    public void setzUzlu(Aparat zUzlu) {
        this.zUzlu = zUzlu;
    }
/**
 * Vrací cílový aparát, kam proud směřuje
 * @return Aparat
 */
    public Aparat getDoUzlu() {
        return doUzlu;
    }
/**
 * Nastavuje cílový aparát, kam proud směřuje
 * @param doUzlu 
 */
    public void setDoUzlu(Aparat doUzlu) {
        this.doUzlu = doUzlu;
    }
/**
 * Vrací textový název proudu
 * @return název proudu
 */
    public String getNazProudu() {
        return nazProudu;
    }
/**
 * Nastavuje textový název proudu
 * @param nazProudu 
 */
    public void setNazProudu(String nazProudu) {
        this.nazProudu = nazProudu;
    }
/**
 * Vrací hmotový průtok proudu (kg/s)
 * @return hmotový průtok
 */
    public double getHmotPrutok() {
        return hmotPrutok;
    }
/**
 * Nastavuje hmotový průtok proudu (kg/s)
 * @param hmotPrutok 
 */
    public void setHmotPrutok(double hmotPrutok) {
        this.hmotPrutok = hmotPrutok;
    }
/**
 * Vrací vstupní teplotu proudu (°C)
 * @return Tim(°C)
 */
    public double gettVstup() {
        return tVstup;
    }
/**
 * Nastavuje vstupní teplotu proudu (°C)
 * @param tVstup 
 */
    public void settVstup(double tVstup) {
        this.tVstup = tVstup;
    }
/**
 * Vrací výstupní teplotu proudu (°C)
 * @return Tout(°C)
 */
    public double gettVystup() {
        return tVystup;
    }
/**
 * Nastavuje výstupní teplotu proudu (°C)
 * @param tVystup 
 */
    public void settVystup(double tVystup) {
        this.tVystup = tVystup;
    }
/**
 * Vrací vstupní tlak proudu (Pa)
 * @return Pin(Pa)
 */
    public double getpVstup() {
        return pVstup;
    }
/**
 * Nastavuje vstupní tlak proudu (Pa)
 * @param pVstup 
 */
    public void setpVstup(double pVstup) {
        this.pVstup = pVstup;
    }
/**
 * Vrací výstupní tlak proudu (Pa)
 * @return Pout(Pa)
 */
    public double getpVystup() {
        return pVystup;
    }
/**
 * Nastavuje výstupní tlak proudu (Pa)
 * @param pVystup 
 */
    public void setpVystup(double pVystup) {
        this.pVystup = pVystup;
    }
/**
 * Vrací hodnotu tlakové ztráty (Pa)
 * @return tlaková ztráta
 */
    public double getTlakZtrata() {
        return tlakZtrata;
    }
/**
 * Nastavuje hodnotu tlakové ztráty (Pa)
 * @param tlakZtrata 
 */
    public void setTlakZtrata(double tlakZtrata) {
        this.tlakZtrata = tlakZtrata;
    }
    
/**
 * Vrací objekt směsi daného proudu
 * @return Smes
 */
    public Smes getSmes() {
        return sm;
    }
/**
 * Nastavuje objekt směsi daného proudu
 * @param sm 
 */
    public void setSmes(Smes sm) {
        this.sm = sm;
    }
/**
 * Vrací pole hmotnostního složení proudu
 * @return xHmot[]
 */
    public double[] getxHmot() {
        return xHmot;
    }
/**
 * Nastavuje pole hmotnostního složení proudu
 * @param xHmot 
 */
    public void setxHmot(double[] xHmot) {
        this.xHmot = xHmot;
    }
/**
 * Vrací pole molárního složení proudu
 * @return xMol[]
 */
    public double[] getxMol() {
        return xMol;
    }
/**
 * Nastavuje pole molárního složení proudu
 * @param xMol 
 */
    public void setxMol(double[] xMol) {
        this.xMol = xMol;
    }
/**
 * Vrací pomocnou proměnnou
 * @return 
 */
    public double getPom() {
        return pom;
    }
/**
 * Nastavuje pomocnou proměnnou
 * používá se např. u batch aparátů
 * @param pom 
 */
    public void setPom(double pom) {
        this.pom = pom;
    }
    
/**
 * Vrací odkaz na seznam elementů proudu
 * @return elementy<Element>
 */
    public List<Element> getTrubky() {
        return prvky;
    }
/**
 * Nastavuje odkaz na seznam elementů proudu
 * @param trubky 
 */
    public void setTrubky(List<Element> trubky) {
        this.prvky = trubky;
    }
/**
 * Vrací textovou proměnnou výsledků proudu
 * @return 
 */
    public String getVysledek() {
        return vysledek;
    }
/**
 * Nastavuje textovou proměnnou výsledků proudu
 * @param vysledek 
 */
    public void setVysledek(String vysledek) {
        this.vysledek = vysledek;
    }
/**
 * Vrací počet elementů proudu
 * @return 
 */
    public int getnElem() {
        return nElem;
    }
/**
 * Vrací zda se vytváří report
 * @return ano/ne
 */
    public boolean isTisk() {
        return tisk;
    }
/**
 * Nastavuje zda se vytváří report
 * @param tisk 
 */
    public void setTisk(boolean tisk) {
        this.tisk = tisk;
    }
/**
 * Vrací příznak spočtení
 * @return 
 */
    public boolean isSpocten() {
        return spocten;
    }
/**
 * Nastavuje příznak spočtení
 * @param spocten 
 */
    public void setSpocten(boolean spocten) {
        this.spocten = spocten;
    }
/**
 * Vrací hodnotu pomocné proměnné
 * @return 
 */
    public int getTag() {
        return tag;
    }
/**
 * Nastavuje hodnotu pomocné proměnné
 * @param tag 
 */
    public void setTag(int tag) {
        this.tag = tag;
    }
    
/**
 * Vrací verzi třídy
 * @return String
 */
    public String getVERZE() {
        return VERZE;
    }
    
}
