/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package cheminzlib;

import fydat.I_Newt;
import fydat.Numer;
import fydat.NumerException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import javax.swing.JOptionPane;

/**
 * Třída pro chemickou rovnováhu soustavy reakcí<br>
 * Rovnovážná konstanta je zadávaná přes třídu <b>Reakce</b><br> spolu se svojí 
 * referenční teplotou (K). Tato třída počítá s kolekcí chemických reakcí,
 * které se účastní rovnováhy.<br>
 * 1- koncentrace; 2-parc. tlak; 3-molární zlomek 
 * @author V. Hlubuček
 */
public class Rovnovaha  implements I_Newt {

    public final double TABS = 273.15;
    protected String VERZE = "V1.3";	  // verze třídy
    private String[] rovnovaha = {"koncentrace - std. stav jednotkové koncentrace", 
        "parciální tlaky - std. stav čistá složka ve stavu id.pl. při 1 atm",
        "molární zlomky - čistá sl. při teplotě a stavu soustavy"};
    /**
     * Koncentrační typ: 1- koncentrace; 2-parc. tlak; 3-molární zlomek
     */
    protected int typC;       // koncentrační typ: 1- koncentrace; 2-parc. tlak; 3-molární zlomek
    /**
     * Pole určující pořadový index reagující složky ve směsi
     */
    protected int[] iPor;     // pole určující pořadový index reagující složky ve směsi
    protected int nSlozReagujicich; // počet reagujících složek směsi
    /**
     * Koncentrační proměnná (koncentrace, parciální tlaky, mol. zlomky)
     */
    protected double[] c;     // koncentrační proměnná (koncentrace, parciální tlaky, mol. zlomky)
    protected double[] c0;    // počáteční koncentrační proměnná (koncentrace, parciální tlaky, mol. zlomky)
    protected double[] x;     // pomocné složení
    /**
     * Rovnovážná konstanta při teplotě t
     */
    protected double kRov;    // rovnovážná konstanta při t 
    protected double t;       // absolutní teplota (K)
    protected double xxPoc;   // počáteční odhad pro konverze
    protected String report;  // string výsledků
    protected Proud pr;       // proud ve kterém se rovnováha odehrává
    protected List<? extends Reakce> listReakci = new ArrayList<Reakce>();      // seznam reakcí

/**
 * Konstruktor třídy Rovnováha
 * @param lst seznam reakcí List<Reakce>
 * @param c   pole počátečních koncentrací double[]
 * @param typ koncentrační typ: 1- koncentrace; 2-parc. tlak; 3-molární zlomek
 * @param pr  proud kde se rovnováha odehrává Proud
 */    
    public Rovnovaha(List<Reakce> lst, double[] c, int typ, Proud pr) {
        this.listReakci = lst;
        this.c = c;
        typC =typ;   // koncentrační typ: 1- koncentrace; 2-parc. tlak; 3-molární zlomek
        this.pr = pr;   // proud kde se rovnováha odehrává
        report = "Rovnováha\n";
    }
    public Rovnovaha() {
        report = "Rovnováha\n";
    }
/**
 * Výpočet chemické rovnováhy soustavy reakcí pro zadané vstupní složení c[].
 * Rovnovážné složení vrací v poli c[], složení směsi proudu vrací na původní.
 * @param typ   koncentrační typ: 1- koncentrace; 2-parc. tlak; 3-molární zlomek
 * @param c     koncentrační proměnná (pole)
 * @param t     teplota absolutní (K)
 * @throws fydat.NumerException, java.io.IOException 
 */
    public int spoctiRovnovahu(int typ, double[] c, double t) throws NumerException, IOException {

        int i, j, nSloz = pr.getSmes().getnSloz(), nReakci = listReakci.size();
        int[] pom = new int[nReakci];
        double[] xx = new double[nReakci];      // pole neznámých konverzí
        double[] xPuv = new double[nSloz];      // záloha původního složení
        typC = (Math.abs(typ) < 4)? typ: 1;     // omezení rozsahu -3 až +3
        if (c.length != nSloz) {
            JOptionPane.showMessageDialog(null, "Chem. rovnováha - nesouhlasí dimenze c s počtem složek!", "Chem. rovnováha", JOptionPane.ERROR_MESSAGE);
            report += "Chyba: nesouhlasí dimenze c s počtem složek!";
            return 0;
        }
        xPuv = pr.getSmes().getxMol();  // zálohované původní molární složení
        for (i = 0; i < nReakci; i++) {
            pom[i] =i;
            xx[i] = (xxPoc > 0)?xxPoc :3000;
        }
        // počet reagujících složek
        HashSet<Integer> reagSl = new HashSet();    // aby byl seznam jedinečných složek
        for (Reakce r : listReakci) {
            report += r.getNazevReakce() + String.format("%nk0= %6.2e  t0(K)= %4.1f  Eakt(kJ/mol)= %5.2f  dH(kJ/mol)= %6.2f%nKrovn(TR0)=%8.5g  TR0(K)= %4.1f  typRov= %1d%n", 
                    r.getK0(),r.getT0(),r.geteAkt()/1000,r.getdH()/1000,r.kR0, r.tR0, r.rTyp);
            for (i = 0; i < r.getiKinSloz().length;i++) {
               report += String.format("Sl.kin. = %3d  řád= %4.2f%n", r.getiKinSloz()[i],r.getRadSlozky()[i]);
            }
            for (j = 0; j < r.getiSlozky().length; j++) {
                int jj;
                jj = r.getiSlozky()[j];  
                if(jj != 0) {
                    reagSl.add(jj);  // přidají se jen dosud neexistující a nenulové prvky (Hash set)
//                    System.out.println("reasl: "+jj+"  j= "+j);
                    report += String.format("Sl.reak, stech : %4d  %2d  %s%n",jj, r.iStech[j], pr.getSmes().getNazevSlozky(pr.getSmes().porCisloLatky(Math.abs(jj)))); 
                }
            }
            report += String.format("(Kladně produkty, záporně reaktanty)%n");
        }
        iPor = new int[reagSl.size()];    // pole s indexy neznámých (reagujících složek)
        nSlozReagujicich = reagSl.size();        // počet složek vstupujících do reakcí

        c0 = new double[c.length];
        double s0 = 0;
        for(i = 0;i < c.length;i++) {c0[i] = c[i]; s0 += c[i];}
        if(3 == typC) for(i = 0; i < c.length; i++){double sc = pr.getSmes().sHust(t, pr.pVstup)/pr.getSmes().sMolHmot(); c0[i] *= sc; c[i] =c0[i];}        
        i = 0;
        for (Integer ii : reagSl) {
            iPor[i] = pr.getSmes().porCisloLatky(Math.abs(ii));
            i++;
        }
        t = pr.tVstup + TABS;
        int stav = new Numer().Newton(this, nReakci, nReakci, nReakci, pom, xx, 0.000001,0.1);    // řešení ---------------------------------
        if (stav == -1) {
            JOptionPane.showMessageDialog(null, "Chem. rovnováha - singularita!", "Chem. rovnováha", JOptionPane.ERROR_MESSAGE);
            report += "*** SINGULARITA! Výsledky nepoužitelné. Zkuste lepší odhad.\n";
        }
        if (stav == -2) {
            JOptionPane.showMessageDialog(null, "Chem. rovnováha - divergence!", "Chem. rovnováha", JOptionPane.ERROR_MESSAGE);
            report += "*** DIVERGENCE! Výsledky nepoužitelné.  Zkuste lepší odhad.\n";
        }
        if (stav > 0) {
            report += "Počet iterací = " + stav + "   (jen rovnováha, neovlivňuje složení proudu)\n"; 
            // Výsledky
            String cc, cr;
            switch (Math.abs(typC)) {
                case 1: cc = " c"; cr = "(mol/m3)"; break;
                case 2: cc = "pi"; cr = "(Pa)"; break;
                case 3: cc = " c"; cr = "(mol/m3)"; break;
                default: cc = " ?"; cr = "?"; break;    
            }
            report += "Typ rovnováhy: " + typC + "   " + rovnovaha[typC - 1] + "\n";
            report += String.format("Rovn.konst = %8.5g   při teplotě (°C, K):%5.1f  %5.1f%n",kRov, t - TABS, t);
            for (i = 0; i < listReakci.size();i++) {
               report += String.format("Reakce  %d  rozsah reakce=  %9.3f " + cr + "  %6.3f%n",i + 1 ,xx[i], xx[i]/(c0[iPor[3]]/Math.abs(stechKoef(iPor[3],i))) ); 
            }
            report += "Rovnovážné složení:\n";
            report += "               složka            " + cc + "0          " + cc + "    x0_mol   x_mol    x_hmot \n";
            double s = 0; 
            s0 = 0;
            for (i = 0; i < pr.getSmes().getnSloz();i++) {
               s += c[i]; s0 += c0[i]; 
            }
            x = new double[c.length];
            for (i = 0; i < pr.getSmes().getnSloz();i++) {
                x[i] = c[i]/s;
            }
            pr.getSmes().zadatSloz(false, x);   // rovnovážné složení se dosadí do směsi proudu
            for (i = 0; i < pr.getSmes().getnSloz();i++) {
               report += String.format(" %20s       %9.4g   %9.4g   %6.4f   %6.4f   %6.4f%n",
                       pr.getSmes().getNazevSlozky(i), c0[i],c[i],c0[i]/s0, pr.getSmes().getxMol(i),pr.getSmes().getxHmot(i)); 
            }            
            report += String.format("Celk.koncentrace " + cc + cr + " (počátek, rovnováha, rozdíl): %8.5g %8.5g %+8.5g %+4.2f %1s%n",s0, s, s - s0, 100*(s - s0)/s0,"%");
        }
        pr.tVystup = pr.tVstup;
        pr.spocten = true;
        pr.getSmes().zadatSloz(false, xPuv);    // návrat původního složení směsi
        return stav;    // -2 divergence; -1 singularita; 0 jiná chyba; >0 počet iterací
       }

/**
 * Výpočet chemické rovnováhy soustavy reakcí pro směs proudu při jeho parametrech.
 * Rovnovážné složení vrací v rámci složení směi proudu.
 * @throws fydat.NumerException, java.io.IOException 
 */
    public int spoctiRovnovahu() throws NumerException, IOException {

        int i, j, nSloz = pr.getSmes().getnSloz(), nReakci = listReakci.size();
        int[] pom = new int[nReakci];
        double[] xx = new double[nReakci];      // pole neznámých konverzí
        double[] xPuv = new double[nSloz];      // záloha původního složení
        typC = 1;     // typ
        xPuv = pr.getSmes().getxMol();  // zálohované původní molární složení
        for (i = 0; i < nReakci; i++) {
            pom[i] =i;
            xx[i] = (xxPoc > 0)?xxPoc :3000;
        }
        // počet reagujících složek
        HashSet<Integer> reagSl = new HashSet();    // aby byl seznam jedinečných složek
        for (Reakce r : listReakci) {
            report += r.getNazevReakce() + String.format("%nk0= %6.2e  t0(K)= %4.1f  Eakt(kJ/mol)= %5.2f  dH(kJ/mol)= %6.2f%nKrovn(TR0)=%8.5g  TR0(K)= %4.1f  typRov= %1d%n", 
                    r.getK0(),r.getT0(),r.geteAkt()/1000,r.getdH()/1000,r.kR0, r.tR0, r.rTyp);
            for (i = 0; i < r.getiKinSloz().length;i++) {
               report += String.format("Sl.kin. = %3d  řád= %4.2f%n", r.getiKinSloz()[i],r.getRadSlozky()[i]);
            }
            for (j = 0; j < r.getiSlozky().length; j++) {
                int jj;
                jj = r.getiSlozky()[j];  
                if(jj != 0) {
                    reagSl.add(jj);  // přidají se jen dosud neexistující a nenulové prvky (Hash set)
//                    System.out.println("reasl: "+jj+"  j= "+j);
                    report += String.format("Sl.reak, stech : %4d  %2d  %s%n",jj, r.iStech[j], pr.getSmes().getNazevSlozky(pr.getSmes().porCisloLatky(Math.abs(jj)))); 
                }
            }
            report += String.format("(Kladně produkty, záporně reaktanty)%n");
        }
        iPor = new int[reagSl.size()];    // pole s indexy neznámých (reagujících složek)
        nSlozReagujicich = reagSl.size();        // počet složek vstupujících do reakcí

        c0 = new double[nSloz];
        t = pr.tVstup + TABS;   // teplota proudu (K)        
        double s0 = 0;
        for(i = 0;i < nSloz;i++) {
            c[i] = pr.getSmes().getxMol(i) * pr.getSmes().sHust(t, pr.pVstup)/pr.getSmes().sMolHmot();  // dopočet vst.koncentrace směsi
            c0[i] = c[i]; 
            s0 += c[i];
        }
//        if(3 == typC) for(i = 0; i < c.length; i++){double sc = pr.getSmes().sHust(t, pr.pVstup)/pr.getSmes().sMolHmot(); c0[i] *= sc; c[i] =c0[i];}        
        i = 0;
        for (Integer ii : reagSl) {
            iPor[i] = pr.getSmes().porCisloLatky(Math.abs(ii));
            i++;
        }
        int stav = new Numer().Newton(this, nReakci, nReakci, nReakci, pom, xx, 0.000001,0.1);    // řešení ---------------------------------
        if (stav == -1) {
            JOptionPane.showMessageDialog(null, "Chem. rovnováha - singularita!", "Chem. rovnováha", JOptionPane.ERROR_MESSAGE);
            report += "*** SINGULARITA! Výsledky nepoužitelné. Zkuste lepší odhad.\n";
            pr.getSmes().zadatSloz(false, xPuv);    // návrat původního složení směsi
        }
        if (stav == -2) {
            JOptionPane.showMessageDialog(null, "Chem. rovnováha - divergence!", "Chem. rovnováha", JOptionPane.ERROR_MESSAGE);
            report += "*** DIVERGENCE! Výsledky nepoužitelné.  Zkuste lepší odhad.\n";
            pr.getSmes().zadatSloz(false, xPuv);    // návrat původního složení směsi
        }
        if (stav > 0) {
            report += "Počet iterací = " + stav + "\n"; 
            // Výsledky
            String cc, cr;
            switch (Math.abs(typC)) {
                case 1: cc = " c"; cr = "(mol/m3)"; break;
                case 2: cc = "pi"; cr = "(Pa)"; break;
                case 3: cc = " c"; cr = "(mol/m3)"; break;
                default: cc = " ?"; cr = "?"; break;    
            }
            report += "Typ rovnováhy: " + typC + "   " + rovnovaha[typC - 1] + "\n";
            report += String.format("Rovn.konst = %8.5g   při teplotě (°C, K):%5.1f  %5.1f%n",kRov, t - TABS, t);
            for (i = 0; i < listReakci.size();i++) {
               report += String.format("Reakce  %d  rozsah reakce=  %9.3f " + cr + "  %6.3f%n",i + 1 ,xx[i], xx[i]/(c0[iPor[3]]/Math.abs(stechKoef(iPor[3],i))) ); 
            }
            report += "Rovnovážné složení:\n";
            report += "               složka            " + cc + "0          " + cc + "    x0_mol   x_mol    x_hmot \n";
            double s = 0; 
            s0 = 0;
            for (i = 0; i < pr.getSmes().getnSloz();i++) {
               s += c[i]; s0 += c0[i]; 
            }
            x = new double[c.length];
            for (i = 0; i < pr.getSmes().getnSloz();i++) {
                x[i] = c[i]/s;
            }
            pr.getSmes().zadatSloz(false, x);   // rovnovážné složení se dosadí do směsi proudu
            for (i = 0; i < pr.getSmes().getnSloz();i++) {
               report += String.format(" %20s       %9.4g   %9.4g   %6.4f   %6.4f   %6.4f%n",
                       pr.getSmes().getNazevSlozky(i), c0[i],c[i],c0[i]/s0, pr.getSmes().getxMol(i),pr.getSmes().getxHmot(i)); 
            }            
            report += String.format("Celk.koncentrace " + cc + cr + " (počátek, rovnováha, rozdíl): %8.5g %8.5g %+8.5g %+4.2f %1s%n",s0, s, s - s0, 100*(s - s0)/s0,"%");
        }
        pr.tVystup = pr.tVstup;
        pr.spocten = true;
        return stav;    // -2 divergence; -1 singularita; 0 jiná chyba; >0 počet iterací
       }
    
    
/**
 * Funkce pravých stran rovnic pro Newton rovnováhy
 * @param x     vektor neznámých  double[]
 * @param ff    vektor reziduí pravých stran double[]
 */    
    public void fun(double[] x, double[] ff) {
        int i, k, alfa;
        double s, sc;
        t = pr.tVstup + TABS;
        // výpočet koncentrací
        for(i = 0; i < nSlozReagujicich; i++) {
            c[iPor[i]] = c0[iPor[i]];
            for(k = 0;k < listReakci.size();k++) {  // přes reakce
              switch (typC) {
                  case 1: case -1: case 3: case -3: c[iPor[i]] += x[k]*(double)stechKoef(iPor[i], k); break;     // c
                  case 2: case -2:  c[iPor[i]] += x[k]*(double)stechKoef(iPor[i], k) * 8.314*t; break;  // p
              }    
              if(c[iPor[i]] < 0) c[iPor[i]] = 1e-8;    // pojistka na záporné koncentrace
            }
        }
        k = 0;        
        // koncentrační typ: 1- koncentrace; 2-parc. tlak; 3-molární zlomek (při minusu bez korekce K(t))
        for(Reakce rea : listReakci) {
            s = rea.getkR0() *Math.exp(rea.getdH()/8.314/rea.gettR0()/t*(t - rea.gettR0())); //K(t)
            alfa = 0;
            for (i = 0; i < rea.iSlozky.length; i++) { alfa += rea.iStech[i]; }  
//          korekce termod. konstanty k typu koncentračního údaje            
//            switch (typC) {     // podle koncentračního typu
//                case 1: s *= Math.pow(8.314*t, -alfa); break;    // c  K*(RT) na -alfa
//                case 2: break;          // p  idealní případ
//                case 3: s *= Math.pow(pr.pVstup, -alfa);         // x  K*P na -alfa
//            }
            kRov = s;   // ještě před přepočtem na Kc            
            if(3 == Math.abs(typC) && alfa != 0)s *= Math.pow(pr.pVstup/8.314/t, alfa);            
            sc = 1;
            for (i = 0; i < rea.iSlozky.length; i++) {
               sc *= Math.pow(c[iPor[i]],(double)stechKoef(iPor[i], k));
            }
            ff[k] = sc -s;  // pravé strany rovnic pro Newton
            k++;
        } 
    }    
/**
 * Vrací true, je-li aspoň jedna z reakcí rovnovážnou
 * @return boolean
 */
    public boolean jeRovn() {
        boolean rovn = false;
        for(Reakce r: listReakci) {
            if(r.rTyp > 0) rovn = true;
        }
        return rovn;
    }
    
/**
 * Vrací stechiometrický koeficient i-té složky reakční směsi
 * v j-té reakci
 * @param iSl pořadové číslo složky ve směsi
 * @param jReak pořadové číslo reakce
 * @return stechiometrický koeficient (int)
 */
  private int stechKoef(int iSl, int jReak) {
      int stech = 0;
      int cSloz = pr.getSmes().getCisloSlozky(iSl);
      Reakce r = listReakci.get(jReak);
      for (int i = 0; i < r.getiStech().length; i++) {
//          System.out.print("* cSloz, Stechi: "+cSloz+"  "+r.getiSlozky()[i]);
          if (cSloz == Math.abs(r.getiSlozky()[i])) {stech = r.getiStech()[i]; break;}
      }
//      System.out.println("** stech " + stech);
      return stech;
  }
  
/**
 * Vrací typ koncentračního údaje
 * 1- koncentrace; 2-parc. tlak; 3-molární zlomek
 * @return typ int
 */
    public int getTypC() {
        return typC;
    }
/**
 * Nastavuje typ koncentračního údaje
 * 1- koncentrace; 2-parc. tlak; 3-molární zlomek
 * @param typC int
 */
    public void setTypC(int typC) {
        this.typC = typC;
    }
/**
 * Vrací pole určující pořadový index reagující složky ve směsi
 * @return int[]
 */
    public int[] getiPor() {
        return iPor;
    }
/**
 * Nastavuje pole určující pořadový index reagující složky ve směsi
 * @param iPor int[]
 */
    public void setiPor(int[] iPor) {
        this.iPor = iPor;
    }
/**
 * Vrací koncentrační proměnnou
 * @return double[]
 */
    public double[] getC() {
        return c;
    }
/**
 * Nastavuje koncentrační proměnnou
 * @param c double[]
 */
    public void setC(double[] c) {
        this.c = c;
    }
/**
 * Vrací abs. teplotu (K)
 * @return double
 */
    public double getT() {
        return t;
    }
/**
 * Nastavuje abs. teplotu (K)
 * @param t (K)
 */
    public void setT(double t) {
        this.t = t;
    }
/**
 * Vrací seznam reakcí
 * @return List<Reakce>
 */
    public List<? extends Reakce> getListReakci() {
        return listReakci;
    }
/**
 * Nastavuje seznam reakcí
 * @param listReakci List<Reakce>
 */
    public void setListReakci(List<Reakce> listReakci) {
        this.listReakci = listReakci;
    }
/**
 * Vrací přiřazený proud
 * @return Proud
 */
    public Proud getPr() {
        return pr;
    }
/**
 * Nastavuje přiřazený proud
 * @param pr Proud
 */
    public void setPr(Proud pr) {
        this.pr = pr;
    }
/**
 * Vrací výsledek
 * @return 
 */
    public String getReport() {
        return report;
    }
/**
 * Vrací hodnotu počátečního odhadu rozsahu reakce
 * @return 
 */
    public double getXxPoc() {
        return xxPoc;
    }
/**
 * Nastavuje hodnotu počátečního odhadu rozsahu reakce
 * @param xxPoc double
 */
    public void setXxPoc(double xxPoc) {
        this.xxPoc = xxPoc;
    }
    
/**
 * Vrací verzi třídy
 * @return 
 */
    public String getVERZE() {
        return VERZE;
    }
    
    
}
