/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cheminzlib;

import javax.swing.JOptionPane;

/**
 * Třída aparátu TepelnyVymenik - nesměšuje dva vstupní proudy do výstupního,<br>
 * ale přes teplosměnnou plochu vyměňuje jejich teplo. Předpokládá zadanou výměnnou plochu,<br> 
 * typ směšování (index protiproudnosti) a koeficient prostupu.<br> Na základě 
 * vstupních teplot dopočte výstupní teploty proudů <br>nebo teplosměnnou plochu
 * pokud je dána jedna z výstupních teplot (teplosměnná plocha se zadá = 0). 
 * @author V.Hlubuček
 */
public class TepelnyVymenik  extends Aparat {

 protected String VERZE = "1.1";   
 protected double indPP;            // index protiproudnosti (0-1) 
 protected double koefProstupu;     // koeficient prostupu (W/m2/K)
 protected double teplosmennaPlocha;    // teplosměnná plocha (m2)
    /**
 * Konstruktor tepelného výměníku
 * @param cApar číslo aparátu
 * @param nazevAparatu textový název aparátu
 */
    public TepelnyVymenik(int cApar, String nazevAparatu) {
        super(cApar,nazevAparatu);        
        pocetInProudu = 2;
        pocetOutProudu = 2;
        myInit();        
    }
/**
 * Konstruktor tepelného výměníku
 * @param nazevAparatu
 * @param plocha - teplosměnná plocha (m2)
 * @param koef - koeficient prostupu tepla (W/m2/K)
 * @param index - index protiproudnosti (0-souproud; 1-protiproud)
 */    
        public TepelnyVymenik(String nazevAparatu, double plocha, double koef, double index) {
        super(0,nazevAparatu);       
        pocetInProudu = 2;
        pocetOutProudu = 2;        
        teplosmennaPlocha = plocha;
        koefProstupu = koef;
        indPP = index;
        myInit();         
    }
/**
 * Konstruktor tepelného výměníku
 * @param nazevAparatu
 * @param pr1 první proud
 * @param pr2 druhý proud
 * @param plocha - teplosměnná plocha (m2)
 * @param koef - koeficient prostupu tepla (W/m2/K)
 * @param index - index protiproudnosti (0-souproud; 1-protiproud)
 */    
        public TepelnyVymenik(String nazevAparatu, Proud pr1, Proud pr2, double plocha, double koef, double index) {   
        this(nazevAparatu, plocha, koef, index);
        proudy.add(pr1);
        proudy.add(pr2);
        proudyIn.add(pr1);      // přidá proud do vstupních
        proudyOut.add(pr1.kopirujProud(pr1)); // a současně výstupních proudů výměníku
        proudyIn.add(pr2);      // přidá proud do vstupních
        proudyOut.add(pr2.kopirujProud(pr2)); // a současně výstupních proudů výměníku
        myInit();         
    }
        
/**
 * Bezparametrový konstruktor tepelného výměníku
 */    
    public TepelnyVymenik() {
        super();        
        pocetInProudu = 2;
        pocetOutProudu = 2;        
        nazevAparatu = "Tepelný výměník ";
        myInit();        
    }
    
    
/**
 * Inicializace aparátu
 * popis labelů a default hodnot pro interaktivní zadávání
 */
    public void myInit() {
        lTexty = new String[10];
        tValue = new String[10];
        lTexty[0] = "Tepelný výměník";
        lTexty[1] = "Č.aparátu";
        lTexty[2] = "Název aparátu";
        lTexty[3] = "Počet In proudů";
        lTexty[4] = "Počet Out proudů";
        lTexty[5] = "Teplosměnná plocha (m2)";
        lTexty[6] = "Koef.prostupu (W/m2/K)";
        lTexty[7] = "Index protiproudnosti (0-1)";
        lTexty[8] = ""; // musí být jako ukončovač
        if(koefProstupu > 0) {
            tValue[1] = String.format("%3d",cApar);
            tValue[2] = nazevAparatu;
            tValue[3] = String.format("%2d",pocetInProudu);
            tValue[4] = String.format("%2d",pocetOutProudu);
            tValue[5] = String.format("%5.2f",teplosmennaPlocha);
            tValue[6] = String.format("%5.0f",koefProstupu);
            tValue[7] = String.format("%4.3f",indPP); 
        } else {
            tValue[1] = "0";
            tValue[2] = "Tepelný výměník";
            tValue[3] = "2";
            tValue[4] = "2";
            tValue[5] = "5";
            tValue[6] = "350";
            tValue[7] = "0,5";             
        }
    }
/**
 * Dosadí do proměnných hodnoty z interaktivního zadání
 */    
    public void dosadInteraktivniHodnoty() {
        cApar = (int)doubleFromString(tValue[1]);
        nazevAparatu = tValue[2];
        pocetInProudu = (int)doubleFromString(tValue[3]);
        pocetOutProudu = (int)doubleFromString(tValue[4]);
        teplosmennaPlocha = doubleFromString(tValue[5]);
        koefProstupu = doubleFromString(tValue[6]);
        indPP = doubleFromString(tValue[7]);
    }    
    
 @Override
    public void pridejProud(Proud pr) {
        super.pridejProud(pr);
        proudyIn.add(pr);      // přidá proud do vstupních
        proudyOut.add(pr.kopirujProud(pr)); // a současně výstupních proudů výměníku
    }
    
 @Override
    public void odeberProud(Proud pr) {
        super.odeberProud(pr);
        odeberProudIn(pr);      // odebere proud do vstupních
        odeberProudOut(pr);     // a současně výstupních proudů výměníku
    }


    @Override
    public boolean jeAparatZadan() {
        boolean neni = false;
        if(proudyIn == null) return false; 
        if(koefProstupu == 0 || teplosmennaPlocha == 0 || indPP == 0) neni = true;
        return (!neni);
    }
    
/**
 * Metoda počítá výměnu tepla mezi proudy
 * Je-li zadána výměnná plocha (>0), spočte koncové teploty,
 * je-li zadána jedna koncová teplota, dopočte výměnnou plochu.
 * Kromě průtoků a vlastností proudů musí být také zadán
 * koeficient prostupu tepla (W/m2/K) mezi proudy.
 */    
    public void vymenTeplo() {
        
        double w1,R,P = 0, Z;       // tepelná kapacita_1, poměr kapacit 12, termiocká účinnost, par Z
        double t1 = 0, t2 = 0, q;          // výstupní teplota chladného a teplého proudu (°C), převedené teplo (J)
        usporadatProudy();
        Proud pr1, pr2;         // chladný a teplý proud
        if (proudyIn.get(0).gettVstup() > proudyIn.get(1).gettVstup()) {
            pr2 = proudyIn.get(0);  pr1 = proudyIn.get(1); 
        } else {
            pr1 = proudyIn.get(0);  pr2 = proudyIn.get(1);             
        }
            w1 = pr1.getHmotPrutok()*pr1.getSmes().sCpW(pr1.gettVstup()+TABS+7, pr1.getpVstup());
            R = w1/pr2.getHmotPrutok()/pr2.getSmes().sCpW(pr2.gettVstup()+TABS-7, pr2.getpVstup());
            Z = Math.sqrt((1+R)*(1+R)-4*R*indPP);
            if (Z<1e-8 && Math.abs(indPP-1)<1e6) Z=1e-5;    // R=1 a protiproud
            report = "Tepelný výměník č. " + cApar + "  " + nazevAparatu + "\n";
            
            if (koefProstupu*teplosmennaPlocha > 0) {   // úloha na koncové teploty
                P = 2/(1+R+Z*Math.cosh(koefProstupu*teplosmennaPlocha/2/w1*Z)/
                        Math.sinh(koefProstupu*teplosmennaPlocha/2/w1*Z));  // termická účinnost
                t1 = pr1.gettVstup() + P *(pr2.gettVstup() - pr1.gettVstup()); // výstupní teplota chladného
                t2 = pr2.gettVstup() - R * (t1 - pr1.gettVstup());      // výstupní teplota teplého
                report += "úloha na  koncové teploty\n";
            }
            if (teplosmennaPlocha <= 0) {   // úloha na teplosměnnou plochu
                if (pr1.gettVystup()>0) {
                   q = w1*((t1=pr1.gettVystup())-pr1.gettVstup());
                   P = (t1 - pr1.gettVstup())/(pr2.gettVstup()-pr1.gettVstup());
                   double ctgh = (2/P-(1+R))/Z;
                   double pom = 0.5*Math.log((ctgh+1)/(ctgh-1));
                   teplosmennaPlocha = 2*w1*pom/Z/koefProstupu;
                   t2 = pr2.gettVstup()-q*R/w1;
                }else {
                   if (pr2.gettVystup()<=0) {
                        JOptionPane.showMessageDialog(null,
                        "Jeden z proudů musí mít zadánu koncovou teplotu (°C)! " ,
                         "Tepelný výměník - úloha na plochu", JOptionPane.WARNING_MESSAGE);
                        return;
                   } 
                   q = w1*(t2=pr2.gettVystup()-pr2.gettVstup());
                   t1 = pr1.gettVstup()+q/w1;
                   P = (t1 - pr1.gettVstup())/(pr2.gettVstup()-pr1.gettVstup());
                   double ctgh = (2/P-(1+R))/Z;
                   double pom = 0.5*Math.log((ctgh+1)/(ctgh-1));
                   teplosmennaPlocha = 2*w1*pom/Z/koefProstupu;                
                }
                report += "úloha na teplosměnnou plochu\n";
                
            }
        if(!proudy.isEmpty()) {           
            if (proudy.get(0).gettVstup() > proudy.get(1).gettVstup()) {
                proudy.get(0).settVystup(t2);  proudy.get(1).settVystup(t1); 
            } else {
                proudy.get(0).settVystup(t1);  proudy.get(1).settVystup(t2); 
            }
        }
        report += "Proud studený: " + pr1.getNazProudu()+ " č. " + pr1.cProudu + 
                String.format("  m(kg/s, kg/h): %6.4g; %6.4g%n",pr1.getHmotPrutok(),pr1.getHmotPrutok()*3600);
        report += String.format(" Vstupní teplota (°C): %5.1f  výstupní teplota (°C): %5.1f%n",
            pr1.gettVstup(), t1);
        double ts;
        ts = (pr1.gettVstup()+t1)/2;    // střední teplota proudu 1
        report += String.format(" Hustota (kg/m3): %7.3f  Cp (J/kg/K): %5.1f%n",
            pr1.getSmes().sHust(ts+TABS, pr1.getpVstup()),pr1.getSmes().sCpW(ts+TABS, pr1.getpVstup()));
        int[] csl = pr1.getSmes().getCislaSlozek();
        report += " Čísla složek: ";
        for (int i = 0; i< csl.length; i++) report += " "+csl[i];
        report += "\n";
        report += "Proud teplý: " + pr2.getNazProudu()+ " č. " + pr2.cProudu +  
                String.format("  m(kg/s, kg/h): %6.4g; %6.4g%n",pr2.getHmotPrutok(),pr2.getHmotPrutok()*3600);
        report += String.format(" Vstupní teplota (°C): %5.1f  výstupní teplota (°C): %5.1f%n",
            pr2.gettVstup(), t2);
        ts = (pr2.gettVstup()+t2)/2;// střední teplota proudu 2
        report += String.format(" Hustota (kg/m3): %7.3f  Cp (J/kg/K): %5.1f%n",
            pr2.getSmes().sHust(ts+TABS, pr2.getpVstup()),pr2.getSmes().sCpW(ts+TABS, pr2.getpVstup()));
        csl = pr2.getSmes().getCislaSlozek();
        report += " Čísla složek: ";
        for (int i = 0; i< csl.length; i++) report += " "+csl[i];
        report += "\n";        
        q = w1*(t1 - pr1.gettVstup());
        report += String.format("Převedené teplo (kW): %7.4g  termická účinnost P:  %5.4f%n",
            q/1000,P);  
        report += String.format("Poměr kapacit R (st/tep): %5.3f   W1(st)(kJ): %6.1f   kS/W1:%7.4f%n", 
                R, w1/1000, koefProstupu*teplosmennaPlocha/w1);        
        report += String.format("Teplosměnná plocha (m2):  %6.2f   koef.prostupu (W/m2/K): %5.0f%n",
                teplosmennaPlocha,koefProstupu);
        ts = (pr2.gettVstup()-indPP>0.9?t1:pr1.gettVstup()) - (t2-indPP>0.9?pr1.gettVstup():t1);
        ts = ts/Math.log((pr2.gettVstup()-indPP>0.9?t1:pr1.gettVstup())/(t2-indPP>0.9?pr1.gettVstup():t1));
        report += String.format("Index protiproudnosti: %4.3f  deltaTln(K): %4.1f   Fk: %4.3f%n",
                indPP, ts, q/koefProstupu/teplosmennaPlocha/ts);
        pr1.spocten = true;
        pr2.spocten = true;
        pr1.zUzlu = this;
        pr2.zUzlu = this;  // proudy pr1 a pr2 jsou již součástí kolekce proudy 
        pr1.nazProudu = "Proud 1 z " + nazevAparatu;
        pr2.nazProudu = "Proud 2 z " + nazevAparatu;
        pr1.setVysledek(report);
        pr2.setVysledek(report);
        {   // blok zapíše výsledné proudy do kolekce proudyOut
            Proud pom;
            // vymazat výstupní proudy před znovu zapsáním
            proudyOut.clear();
            if(!proudy.isEmpty()) {
                if (proudy.get(0).gettVstup() > proudy.get(1).gettVstup()) {
                    pom = pr2.kopirujProud(pr2);
                    pom.tVstup = pom.tVystup;   // výsledná teplota je u výstupního prpudu vstupní
                    pridejProudOut(pom);
                    pom = pr1.kopirujProud(pr1);
                    pom.tVstup = pom.tVystup;                
                    pridejProudOut(pom);
                } else {
                    pom = pr1.kopirujProud(pr1);
                    pom.tVstup = pom.tVystup;   // výsledná teplota je u výstupního prpudu vstupní
                    pridejProudOut(pom);
                    pom = pr2.kopirujProud(pr2);
                    pom.tVstup = pom.tVystup;                
                    pridejProudOut(pom);
                }
            }
        }
        spocten = true;
        if(zobrazujVysledky) {
            new JDialogProud(null, "Tepelný výměník proud 1 (Verze " 
                        + VERZE + ")\n", pr1); // modálně vyskakovací  
            new JDialogProud(null, "Tepelný výměník proud 2 (Verze " 
                        + VERZE + ")\n", pr2); // modálně vyskakovací             
        }
}
    
/**
 * implementace abstraktní metody
 */
    @Override
    public void spoctiAparat() {
        vymenTeplo();
    }

    @Override
    public void spoctiAparat(boolean tisk) {
        boolean bylo = zobrazujVysledky;
        if(tisk)zobrazujVysledky = true; else zobrazujVysledky = false;
        vymenTeplo();
        zobrazujVysledky = bylo;
    }

 
    
// gettery a settery  -------------------------------------------------------    
/**
 * Vrací index protiproudnosti (0-1)
 * @return index
 */
    public double getIndPP() {
        return indPP;
    }
/**
 * Nastavuje index protiproudnosti (0-1)
 * @param indPP 
 */
    public void setIndPP(double indPP) {
        this.indPP = indPP;
    }
/**
 * Vrací hodnotu koeficientu prostupu tepla (W/m2/K)
 * @return koef. prostupu
 */
    public double getKoefProstupu() {
        return koefProstupu;
    }
/**
 * Nastavuje hodnotu koeficientu prostupu tepla (W/m2/K)
 * @param koefProstupu 
 */
    public void setKoefProstupu(double koefProstupu) {
        this.koefProstupu = koefProstupu;
    }
/**
 * Vrací hodnotu teplosměnné plochy (m2)
 * @return plocha
 */
    public double getTeplosmennaPlocha() {
        return teplosmennaPlocha;
    }
/**
 * Nastavuje hodnotu teplosměnné plochy (m2)
 * @param teplosmennaPlocha 
 */
    public void setTeplosmennaPlocha(double teplosmennaPlocha) {
        this.teplosmennaPlocha = teplosmennaPlocha;
    }
    
}
