/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fydat;

/**
 * Základní numerické metody (minimalizace, lin. rovnice, Newton, ODE)
 * obsahuje statické metody a také některé jejich instanční ekvivalenty
 * pro případ vícenásobného volání.
 * Součást knihovny (balíku) fydat.<br>
 * m_Gauss a Gauss - řešení soustavy lineárních rovnic (statická a instanční)<br>
 * m_Fmin, m_Fmin1 - minimalizace funkce n-proměnných Rosenbrock a NelderMead (obě statické)<br>
 * m_RKM - řešení soustavy obyč. dif. rovnic prvního řádu s automatickou regulaci integračního 
 * kroku Runge-Kutta-Merson (4. řád, statická)<br>
 * @author Hlubuček 05/2010, 2014
 */
public class Numer {
    // Nastavitelné parametry metod (i z aplikačních programů)

    private  static boolean debugPrint = false;    // potlačuje kontrolní výpisy
    private  static double stp = 0;         // zapamatovaný integrační krok pro RKM
    private  static int itmaxFmin = 300;    // maximální počet iterací Fmin
    private  static double E0Fmin = 0.05;   // expanzní faktor kroku první souřadnice
    private  static double E1Fmin = 0.02;   // expanzní faktor kroku dalších souřadnic
    private  static int iterFmin = 0;       // skutečný počet iterací po výstupu z Fmin
    private  static int itmaxNewt = 300;  // max. počet iterací pro Newton
/**
 * Proměnná vrací hodnotu zda je nastaven tisk ladicích tisků (true)
 * @return boolean
 */
    public boolean isDebugPrint() {
        return debugPrint;
    }
/**
 * Proměnná nastavuje hodnotu pro tisk ladicích tisků (true)
 * @param debugPrint - boolaen
 */
    public void setDebugPrint(boolean debugPrint) {
        this.debugPrint = debugPrint;
    }
/**
 * Vrací nastavení expanzního kroku 1. souř.metody Fmin
 * @return double
 */
    public double getE0Fmin() {
        return E0Fmin;
    }
/**
 * Nastavuje hodnotu expanzního kroku 1. souř. metody Fmin
 * @param E0Fmin
 */
    public void setE0Fmin(double E0Fmin) {
        this.E0Fmin = E0Fmin;
    }
/**
 * Vrací nastavení expanzního kroku ost. souř.metody Fmin
 * @return
 */
    public  double getE1Fmin() {
        return E1Fmin;
    }
/**
 * Nastavuje hodnotu expanzního kroku ost. souř. metody Fmin
 * @param E1Fmin
 */
    public  void setE1Fmin(double E1Fmin) {
        this.E1Fmin = E1Fmin;
    }
/**
 * Vrací počet provedených iterací metody Fmin a Fmin1
 * @return IterFmin int
 */
    public static int getIterFmin() {
        return iterFmin;
    }
/**
 * Vrací nastavení max. počtu iterací pro metody Fmin a Fmin1
 * @return ItmaxFmin int
 */
    public int getItmaxFmin() {
        return itmaxFmin;
    }
/**
 * Provádí nastavení max. počtu iterací pro metody Fmin a Fmin1
 * @param itmaxFmin int
 */
    public void setItmaxFmin(int itmaxFmin) {
        this.itmaxFmin = itmaxFmin;
    }
/**
 * Vrací nastavení max. počtu iterací pro Newton
 * @return 
 */
    public static int getItmaxNewt() {
        return itmaxNewt;
    }
/**
 * Provádí nastavení max. počtu iterací pro Newton
 * @param itmaxNewt 
 */
    public static void setItmaxNewt(int itmaxNewt) {
        Numer.itmaxNewt = itmaxNewt;
    }

    /**
     * Integrace ODE Runge-Kutta-Merson s automatickou regulací délky kroku - statická metoda
     * @author Hlubuček
     * @param ff podprogram pravých stran rovnic dle interface I_RKM
     * @param n    počet integrovaných rovnic
     * @param xpoc, xkon   interval integrace
     * @param y[]  pole integrovaných proměnných
     * @param accur  absolutní požadovaná přesnost
     * @param start    musí být true jen v počátečním kroku
     * @param z[]  pole parametrů ODE
     */
    public static void m_RKM(I_RKM ff, int n, double xpoc, double xkon, double[] y,
            double accur, boolean start, double[] z) {
        // Deklarace
        final double HMIN = 1E-5;   // minimální hodnot aintegračního kroku
        final int N = n;
        double h = HMIN;               // integrační krok
        boolean last;
        //double stp=0;      // zapamatovaný integrační krok
        double eps, h3, x0, q;         // vyp. přesnost a pom. proměnné
        double[] y0 = new double[N];
        double[] ak1 = new double[N];
        double[] ak2 = new double[N];
        double[] ak3 = new double[N];
        double[] ak4 = new double[N];
        double[] ak5 = new double[N];

        if (accur < 1E-8) {
            accur = 1E-5;
        }
        if (!start) {
            h = stp;     // není počáteční krok
        }//        System.out.println("x,stp: "+xpoc+" "+stp);
        do {
            q = Math.abs(h);
            if (Math.abs(xkon - xpoc) > Math.abs(h)) {
                last = false;
            } else {
                last = true;
                h = xkon - xpoc;
            }
            x0 = xpoc;
            for (int i = 0; i < n; i++) {
                y0[i] = y[i];
            }
            h3 = h / 3;
            ff.fkt(n, xpoc, y, ak1, z);
            for (int i = 0; i < n; i++) {
                y[i] = ak1[i] * h3 + y0[i];
            }
            xpoc = x0 + h3;
            ff.fkt(n, xpoc, y, ak2, z);
            for (int i = 0; i < n; i++) {
                y[i] = (ak1[i] + ak2[i]) / 2 * h3 + y0[i];
            }
            ff.fkt(n, xpoc, y, ak3, z);
            for (int i = 0; i < n; i++) {
                y[i] = (0.375 * ak1[i] + 1.125 * ak2[i]) * h3 + y0[i];
            }
            xpoc = 0.5 * h + x0;
            ff.fkt(n, xpoc, y, ak4, z);
            for (int i = 0; i < n; i++) {
                y[i] = (1.5 * ak1[i] - 4.5 * ak3[i] + 6 * ak4[i]) * h3 + y0[i];
            }
            xpoc = x0 + h;
            ff.fkt(n, xpoc, y, ak5, z);
            for (int i = 0; i < n; i++) {
                y[i] = (0.5 * (ak1[i] + ak5[i]) + 2 * ak4[i]) * h3 + y0[i];
            }
            eps = 0;
            for (int i = 0; i < n; i++) {
                q = Math.abs(0.2 * ak1[i] - 0.9 * ak3[i] + 0.8 * ak4[i] - 0.1 * ak5[i]);
                if (q > eps) {
                    eps = q;
                }
            }
            eps = eps + Math.abs(h3);
//        System.out.println("xpoc, y: "+xpoc+" "+y[0]+" "+h+" "+eps);
            if (accur < eps) {
                for (int i = 0; i < n; i++) {
                    y[i] = y0[i];
                }
                xpoc = x0;
            } else {
                if (last) {
                    if (start) {
                        stp = h;
                    }
//                    System.out.println("xk,stp: "+xpoc+" "+stp);
                    return;
                } else {
                    if (eps > 0) {
                        h = Math.pow(accur / eps, 0.2) * 0.8 * h;
                        if (Math.abs(h) < HMIN) {
                            h = HMIN * StrictMath.signum(h);
                        }
                        stp = h;
                    }
                    start = false;
                }
            }
        } while (accur >= eps);
    } // konec RKM

     /**
     * Řešení soustavy lin. rovnic Gaussovou eliminací - statická metoda
     * @param n počet rovnic
     * @param a rozšířená matice soustavy
     * @param b vektor řešení
     * @return stav 1 OK, -1 singularita
     */
    public static int m_Gauss(int n, double[][] a, double[] b) {
        int i, j, imax = 0, stav = 0;
        double amax, aa;

        // přímý chod
        for (i = 0; i < n; i++) {
            // hledání max. prvku v sloupci
            amax = 0;
            for (j = 0; j < n; j++) {
                aa = Math.abs(a[j][i]);
                if (aa > amax) {
                    amax = aa;
                    imax = j;
                }
            }
            if (amax < 1E-7) {
                stav = -1;
                return stav;
            }   // singularita
            amax = a[imax][i];
            // výměna řádků
            if (i != imax) {
                for (j = 0; j < n + 1; j++) {
                    aa = a[i][j];
                    a[i][j] = a[imax][j];
                    a[imax][j] = aa;
                }
            }
            // Dělení pivotem
            for (j = 0; j < n + 1; j++) {
                a[i][j] = a[i][j] / amax;
            }
            // Eliminace od řádku i+1
            for (int ii = i + 1; ii < n; ii++) {
                aa = a[ii][i];
                for (j = i; j < n + 1; j++) {
                    a[ii][j] = a[ii][j] - a[i][j] * aa;
                }
            }
        }
        // Zpětný chod
//        System.out.println("Před zpět. chodem");
//        for ( i=0; i<n; i++) {
//            for ( j=0; j<n+1; j++) System.out.print(" "+a[i][j]);
//            System.out.print("\n");
//        }
        for (i = n - 2; i >= 0; i--) {
            for (j = i + 1; j < n; j++) {
                a[i][n] += -a[i][j] * a[j][n];
            }
        }
        // Řešení
        for (i = 0; i < n; i++) {
            b[i] = a[i][n];
        }
        stav = 1;
        return stav;
    } // konec Gauss

     /**
     * Řešení soustavy lin. rovnic Gaussovou eliminací - <b>instanční metoda</b>
     * @param n počet rovnic
     * @param a rozšířená matice soustavy
     * @param b vektor řešení
     * @return stav 1 OK, -1 singularita
     */
    public int Gauss(int n, double[][] a, double[] b) {
        int i, j, imax = 0, stav = 0;
        double amax, aa;

        // přímý chod
        for (i = 0; i < n; i++) {
            // hledání max. prvku v sloupci
            amax = 0;
            for (j = 0; j < n; j++) {
                aa = Math.abs(a[j][i]);
                if (aa > amax) {
                    amax = aa;
                    imax = j;
                }
            }
            if (amax < 1E-7) {
                stav = -1;
                return stav;
            }   // singularita
            amax = a[imax][i];
            // výměna řádků
            if (i != imax) {
                for (j = 0; j < n + 1; j++) {
                    aa = a[i][j];
                    a[i][j] = a[imax][j];
                    a[imax][j] = aa;
                }
            }
            // Dělení pivotem
            for (j = 0; j < n + 1; j++) {
                a[i][j] = a[i][j] / amax;
            }
            // Eliminace od řádku i+1
            for (int ii = i + 1; ii < n; ii++) {
                aa = a[ii][i];
                for (j = i; j < n + 1; j++) {
                    a[ii][j] = a[ii][j] - a[i][j] * aa;
                }
            }
        }
        // Zpětný chod
//        System.out.println("Před zpět. chodem");
//        for ( i=0; i<n; i++) {
//            for ( j=0; j<n+1; j++) System.out.print(" "+a[i][j]);
//            System.out.print("\n");
//        }
        for (i = n - 2; i >= 0; i--) {
            for (j = i + 1; j < n; j++) {
                a[i][n] += -a[i][j] * a[j][n];
            }
        }
        // Řešení
        for (i = 0; i < n; i++) {
            b[i] = a[i][n];
        }
        stav = 1;
        return stav;
    } // konec Gauss


    /**
     * Minimalizace funkce Rosenbrockovou metodou - statická metoda
     * @author Hlubuček
     * @param ff minimalizovaná fce důe I_Fmin
     * @param n počet proměnných
     * @param x[] pole proměnných
     * @param eps abs.přesnost
     * @return hodnota funkce v minimu
     * @throws fydat.NumerException
     */
    public static double m_Fmin(I_Fmin ff,  int n, double[] x, double eps) throws NumerException {
        final int N = n;
        int i, i1, j, jj = 0, jjj = 0;
        double[] a = new double[N];
        double[] d = new double[N];
        double[] e = new double[N];
        double[] r = new double[N];
        double[][] b = new double[N][N];
        double[][] c = new double[N][N];
        double[][] v = new double[N][N];
        double fce, fh, s;

        fce = ff.f(x);
        for (i = 0; i < n; i++) {
            for (j = 0; j < n; j++) {
                v[i][j] = 0;
                if (i == j) {
                    v[i][j] = 1;
                }
            }
        }
        //10:
        for (j = 0; j < n; j++) {
            a[j] = 2;
            d[j] = 0;
            e[j] = E1Fmin / (jj + 1);
        }
        e[0] = E0Fmin / (jj + 1);
        //20:
        i = 0;  // dříve 1
        Akce:
        do {
            for (j = 0; j < n; j++) {
                x[j] = x[j] + e[i] * v[i][j];
            }
            fh = ff.f(x);
            if (fh < fce) {
                d[i] = d[i] + e[i] * (1 + (fce - fh) / (fh + 1E-8));  // úspěch
                e[i] = 3 * e[i];
                fce = fh;
                if (a[i] > 1.5) {
                    a[i] = 1;
                }
                if (debugPrint) {
                    System.out.println("* i=" + i + " fh, x: " + fh + " " + x[0] + " " + x[1] + "  iter=" + jj + " e0, e1: " + e[0] + " " + e[1]);
                }
            } else {  // neúspěch, vrať zpět
                for (j = 0; j < n; j++) {
                    x[j] = x[j] - e[i] * v[i][j];
                }
//                if (debugPrint) {
//                    System.out.println("** i=" + i + " fh, x: " + fh + " " + x[0] + " " + x[1]);
//                }
                e[i] = -0.5 * e[i];   // krok opačným směrem, poloviční
                if (a[i] < 1.5) {
                    a[i] = 0;
                }
            }
            boolean kon = true;
            for (j = 0; j < n; j++) {
                if (Math.abs(e[j]) >= eps) {
                    kon = false;
                }
                if (kon) {
                    iterFmin = jj;
                    return fce;
                }   // splněná přesnost
            }
            jj = jj + 1;
            if (jj > itmaxFmin) {
                iterFmin = jj;
                throw new NumerException("Prekrocen..", fce);
//                JOptionPane.showMessageDialog(null, "Překročen max. počet iterací " + itmaxFmin, "Fmin", JOptionPane.WARNING_MESSAGE);
//                return fce;
            }
            for (j = 0; j < n; j++) {
                if (a[j] > 0.5) {
                    if (i < n - 1) {
                        i = i + 1;
                    } else {
                        i = 0;
                    }
                    continue Akce;  // go to 30
                }
            }
            // Roatce os Gram-Schmidtova ortogonalizace
            if (debugPrint) {
                System.out.println("GS fh, x: " + fh + " " + x[0] + " " + x[1] + "  iter=" + jj);
            }
            for (int l = 0; l < n; l++) {
                for (j = 0; j < n; j++) {
                    c[l][j] = 0;
                    for (int k = l; k < n; k++) {
                        c[l][j] += d[k] * v[k][j];
                    }
                }
            }
            for (int l = 0; l < n; l++) {
                i1 = l - 1;
                if (l > 0) {
                    for (j = 0; j < i1; j++) {
                        r[j] = 0;
                        for (int k = 0; k < n; k++) {
                            r[j] += c[l][k] * v[j][k];
                        }
                    }
                }
                for (j = 0; j < n; j++) {
                    b[l][j] = c[l][j];
                    if (l == 0) {
                        continue;
                    }
                    for (int k = 0; k < i1; k++) {
                        b[l][j] += -r[k] * v[k][j];
                    }
                }
                s = 0;
                for (j = 0; j < n; j++) {
                    s += b[l][j] * b[l][j];
                }
                for (j = 0; j < n; j++) {
                    v[l][j] = b[l][j] / (Math.sqrt(s) + 1e-20);
                }
            }
            // inicializace (goto 10) po ortogonalizaci
            for (j = 0; j < n; j++) {
                a[j] = 2;
                d[j] = 0;
                e[j] = E1Fmin / (Math.pow(jj, 0.4) + 1);
            }
            e[0] = E0Fmin / (Math.pow(jj, 0.4) + 1);
            i = 0;
        } while (true);

    }   // konec Fmin


    /**
     * Minimalizace funkce metodou Nelder-Mead - statická metoda
     * @author Hlubuček
     * @param ff minimalizovaná fce důe I_Fmin
     * @param n počet proměnných
     * @param x[] pole proměnných
     * @param eps abs.přesnost
     * @return hodnota funkce v minimu
     * @throws fydat.NumerException
     */
    public static double m_Fmin1(I_Fmin ff, int n, double[] x, double eps)
            throws NumerException {
        final int N = n;
        int i, j, k1, k2, k3, k4, kount, l, index, iter = 0;
        double alfa, beta, gama, delta, difer, xnx, step, step1, step2, sumi;
        double sumH = 0, sumL = 0, sum2, sums;
        double[][] x1 = new double[20][20];
        double[][] a = new double[20][20];
        double[] sum = new double[20];

        alfa = 1;       // faktor reflexe
        beta = 0.5;     // faktor kontarkce
        gama = 2;       // faktor expanze
        delta = 0.5;    // faktor redukce
        step = 0.1;     // hrana simplexu
        xnx = n;
        sum[1] = ff.f(x);
        k1 = n + 1;     // funguje jako mez cyklu, dimenze řádků simplexu <k1
        k2 = n + 1;     // index řádeku s těžištěm simplexu v x1[][]
        k3 = n + 2;     // index řádeku reflexe
        k4 = n + 3;     // index řádeku expanze
        step1 = step / xnx / 1.41421 * (Math.sqrt(xnx + 1) + xnx - 1);
        step2 = step / xnx / 1.41421 * (Math.sqrt(xnx + 1) - 1);
        // Tvorba simplexu
        for (j = 0; j < n; j++) {
            a[0][j] = 0;
        }
        for (i = 1; i < k1; i++) {
            for (j = 0; j < n; j++) {
                a[i][j] = step2;
                l = i - 1;
                a[i][l] = step1;
            }
        }
        for (i = 0; i < k1; i++) {
            for (j = 0; j < n; j++) {
                x1[i][j] = x[j] + a[i][j];
            }
        }
        for (i = 0; i < k1; i++) {
            for (j = 0; j < n; j++) {
                x[j] = x1[i][j];
            }
            sum[i] = ff.f(x);
        }
        do {    // Začátek hlavního cyklu ---------------------------------
            // Vyber největší hodnotu sum[i] v simplexu
            //28:
            sumH = sum[0];
            index = 0;
            for (i = 1; i < k1; i++) {
                if (sum[i] > sumH) {
                    sumH = sum[i];
                    index = i;
                }
            }
            // Vyber nejmenší hodnotu sum[i] v simplexu
            sumL = sum[0];
            kount = 0;
            for (i = 1; i < k1; i++) {
                if (sumL > sum[i]) {
                    sumL = sum[i];
                    kount = i;
                }
            }
            // Najdi těžiště bodů pro i != index
            for (j = 0; j < n; j++) {
                sum2 = 0;
                for (i = 0; i < k1; i++) {
                    sum2 = sum2 + x1[i][j];
                }
                x1[k2][j] = 1 / xnx * (sum2 - x1[index][j]);
                // Proveď reflexi ( x(k3) = x(k2) + alfa*(x(k2) - x(index))  jiný ekv. tvar
                x1[k3][j] = (1.0 + alfa) * x1[k2][j] - alfa * x1[index][j];
                x[j] = x1[k3][j];
            }
//            System.out.println("Teziste " + x1[k2][0] + " " + x1[k2][1]);
            sum[k3] = ff.f(x);
//            System.out.println("Refl: " + sum[k3] + " x: " + x[0] + " " + x[1]);
            if (sum[k3] >= sumL) {  // reflexe nebyla nejlepší - neúspěšná větev z hlediska reflexe
                // Vyber druhou hodnotu
                if (index != 0) {
                    sums = sum[0];
                } else {
                    sums = sum[1];
                }
                for (i = 0; i < k1; i++) {
                    if (index != i) {
                        if (sum[i] >= sums) {
                            sums = sum[i];
                        }
                    }
                }
                if (sum[k3] > sums) { // reflexe je  horší, než 2. nejhorší bod
                    if (sum[k3] <= sumH) { // ale není úplně nejhorší
                        for (j = 0; j < n; j++) {   // dosaď reflexi na dosud nejhorší
                            x1[index][j] = x1[k3][j];
                        }
                    }
                    for (j = 0; j < n; j++) {   // kontrakce po expanzi
                        // jiný ekv. tvar  x(k4)= x(k2) + beta*(x(index) - x(k2))
                        x1[k4][j] = beta * x1[index][j] + (1 - beta) * x1[k2][j];
                        x[j] = x1[k4][j];
                    }
                    sum[k4] = ff.f(x);
//                    System.out.println("Kontrakce: " + sum[k4] + " x: " + x[0] + " " + x[1]);
                    if (sumH <= sum[k4]) {  // Kontrakce je nejhorší
                        // Redukce simplexu kolem nejlepšího bodu
                        for (j = 0; j < n; j++) {
                            for (i = 0; i < k1; i++) {
                                x1[i][j] = delta * (x1[i][j] + x1[kount][j]);
                            }
                        }
                        for (i = 0; i < k1; i++) {
                            for (j = 0; j < n; j++) {
                                x[j] = x1[i][j];
                            }
                            sum[i] = ff.f(x);
                        }
//                        System.out.println("Redukce: " + x[0] + " " + x[1]);
                    } else {    // nejhorší bod nahrazen kontrakcí
                        //16:
                        for (j = 0; j < n; j++) {
                            x1[index][j] = x1[k4][j];
                            x[j] = x1[index][j];
                        }
                        sum[index] = sum[k4];
                    }

                } else {    // neúspěšná reflexe nebyla úplně nejhorší  - dosaď reflexi za dosud nejhorší bod
                    for (j = 0; j < n; j++) {
                        x1[index][j] = x1[k3][j];
                        x[j] = x1[index][j];
                    }
                    sum[index] = sum[k3];
                }  // pokračovat x=těžiště
            } else {        // --------------------------------------------------
                // 11: Proveď expanzi nového minima...úspěšná větev z hlediska reflexe
                for (j = 0; j < n; j++) {
                    // jiný ekv. tvar x(k4) = x(k2) + gama*(x(k3) - x(k2))
                    x1[k4][j] = (1.0 - gama) * x1[k2][j] + gama * x1[k3][j];
                    x[j] = x1[k4][j];
                }
                sum[k4] = ff.f(x);
//                System.out.println("Expand: " + sum[k4] + " x: " + x[0] + " " + x[1]);
                if (sum[k4] < sumL) {   // Expanze úspěch - dosaď expanzi za dosud nejhorší bod
                    for (j = 0; j < n; j++) {
                        x1[index][j] = x1[k4][j];
                        x[j] = x1[index][j];
                    }
                    sum[index] = sum[k4];
                } else {    // Expanze neúspěch - nejhorší bod nahraď úspěšnou reflexí
                    for (j = 0; j < n; j++) {
                        x1[index][j] = x1[k3][j];
                        x[j] = x1[index][j];
                    }
                    sum[index] = sum[k3];
                }
            }
            //26:  dosaď těžiště
            for (j = 0; j < n; j++) {
                x[j] = x1[k2][j];
            }
            sum[k2] = ff.f(x);
            difer = 0;
            for (i = 0; i < k1; i++) {
                difer += (sum[i] - sum[k2]) * (sum[i] - sum[k2]);
            }
            difer = 1 / xnx * Math.sqrt(difer);     //  /Math.abs(sum[k2]);  přidáno  neúspěch
            iter = iter + 1;
            if (iter > itmaxFmin) {
                iterFmin = iter;
                throw new NumerException("Prekrocen..", sumL);
//               return sumL;
            }
            if (debugPrint && iter % 5 == 0) {
                System.out.println("FMIN1: " + sum[k4] + " x: " + x[0] + " " + x[1] + " * iter: " + iter);
            }
        } while (difer >= eps * eps || iter < 10);  // konec hlavního cyklu  ------------
        iterFmin = iter;
        return sumL;
    } // konec Fmin1


/**
 * Řešení soustavy (i přeurčených) nelineárních rovnic Newtonovou metodou
 * s numerickým výpočtem Jacobianu - <b>instanční metoda</b>
 * @param equat rozhraní I_Newt definující funkci fun pravých stran rovnic
 * @param nEquat počet rovnic pravých stran
 * @param nProm celkový počet proměnných
 * @param nUnknown  počet řešených neznámých
 * @param iPorad[] pole definující pořadové indexy řešených neznámých
 * @param xx[]  pole odhadů/řešení
 * @param eps  přesnost řešení (abs.)
 * @param alam  tlumení
 *   >0 a <2 ..maximální přípustná relativní korekce promenne (doporučeno 0.5)
 *          zachovává stejné znaménko proměnné !
 *   <0  v abs. hodnotě stejný účinek, umožňuje změnu znaménka!
 *   >2  zlomková část alam tvoří tlumicí konstantu pro klasické tlumeni abs.oprav kořenu
 * @return  stav =počet iterací OK; -1 singularita; -2 diverence
 * @throws fydat.NumerException
 */
    public  int Newton(I_Newt equat,int nEquat, int nProm, int nUnknown,
            int[] iPorad, double[] xx, double eps, double alam) throws NumerException {
        int i, j, jj, imax = 0, jmax = 0, iter = 0, stav;

        double aa, d, delta=0.02, tlumx;
        double[] x = new double[nProm], ff = new double[nEquat];
        double[] ffmod = new double[nEquat], b = new double[nUnknown];
        double[][] xJac = new double[nEquat][nEquat+1];
        double[][] xJacMod = new double[nUnknown][nUnknown+1];

        for (i = 0; i<nProm; i++) {   // Počet proměnných použitých v podprogramu fun( )   
          x[i] = xx[i];               //  ze kterých je možno vybírat neznámé
        } 
        equat.fun(x, ff);    // volání funkce pravých stran
        while (iter < itmaxNewt) {
          iter++;
//      generování Jakobiánu          
          for (i = 0; i < nEquat; i++) {
            for (j = 0; j<nEquat + 1; j++) {
                xJac[i][j] = ff[i];
            }
          }
          for (jj = 0; jj< nUnknown; jj++) {
            j = iPorad[jj];
            aa = x[j];
            d = Math.abs(x[j]) * delta;
            if (d < 0.0001) d = 0.0001;
            x[j] = x[j] + d;
            equat.fun(x, ff);    // volání funkce pravých stran
            x[j] = aa;           // navrácení původní hodnoty
            for (i = 0; i< nEquat; i++) {
              xJac[i][jj]= (ff[i] - xJac[i][jj]) / d;
            } 
          }
//        modifikace jakobianu na ctvercovy system -----------------------------
          for (i = 0; i< nUnknown; i++) {
            for (j = 0; j < nUnknown; j++) {
              xJacMod[i][j] = 0;
              for (int m = 0; m < nEquat; m++) {
                xJacMod[i][j] = xJacMod[i][j] + xJac[m][i] * xJac[m][j];
              }
            }
          }
//        modifikace pravych stran na ctvercovy system -------------------------
          for (i = 0; i< nUnknown; i++) {
            ffmod[i] = 0;
            for (int m = 0; m < nEquat; m++) {
              ffmod[i] = ffmod[i] + xJac[m][i] * xJac[m][nUnknown];
            }
          }
          for (i = 0; i< nUnknown; i++) {
            xJacMod[i][nUnknown] = ffmod[i];   // pravé strany
          }          
//      Vypocet netlumene korekce
          stav = Gauss(nUnknown, xJacMod, b);   // používá instanční metodu
          if (stav != 1) {
              // Jacobian singularní !
              return stav;
          }
//   Stanoveni tlumeni
        double amax = 0;
        for (jj = 0; jj< nUnknown; jj++) {
            i = iPorad[jj];
            aa = Math.abs(b[jj]/ x[i]);
            if (aa > amax && x[i] > 0.00001) {
                amax = aa;
                imax = i;
                jmax = jj;
            }
        }
        tlumx = 1;
        if (amax > Math.abs(alam)  && alam < 2) {
         tlumx = Math.abs(alam) / amax;
         if (Math.abs(x[imax]) < 0.01) {
          if (x[imax] * (x[imax] - b[jmax]) < 0) {
            if (alam < 0) {
            x[imax] = 0;         // Prechod reseni pres nulu pri alam<0
            tlumx = 0.5;
            }
          }
         }
        }
        if (alam > 2) {                           
          tlumx = alam - (int)alam;          // Zlomkova cast jako tlumeni
        }        
//    Korekce neznamych
        for (jj = 0; jj< nUnknown; jj++) {
            i = iPorad[jj];
            x[i] = x[i] - tlumx * b[jj];
         }
        if (debugPrint) {
            System.out.println("* iter= "+iter); 
           for (i = 0; i < nUnknown; i++) {
               System.out.println("x"+(i+1)+" = "+x[i]);
           } 
        }
//   Posledni prepocet funkci a konecneho residua
        equat.fun(x, ff);    // volání funkce pravých stran
        if (amax < eps) break;                           
        }
        for (i = 0; i < nUnknown; i++) {
            xx[i] = x[i];       // navrácení řešení do zadávacího pole
        }
        if (iter >= itmaxNewt) {    // divergence       
//            throw new NumerException("Prekrocen..", -9999.);
            return -2;  
        }
        return iter;
    }

    
    
    public static int add(int a, int b) {
        return a + b;
    }
} // konec Numer

