/*
 * Package fydatlib
 * Knihovna metod pro fyzikální vlastnosti org. látek a technických plynů
 * a jejich směsí
 */
package fydatlib;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

/**
 * Třída Latka jako složka směsi - FYDAT<br>
 * Obsahuje bodové vlastnosti a teplotně závislé rozvoje jedné látky<br>
 * Načítá data o látkách z báze FYDAT, soubor <b>fydat.drc</b> 
 * @author V.Hlubuček
 */
public class Latka {

    // duben 2013 odstraněna nutnost zadávat číslo látky zmenšené o 1 do getLatka()
    protected int cislo;       // číslo látky
    protected int group;       // druhová skupina
/**
 * Číslo látky dle Chemical Abstracts
 */    
    protected long cas;        // CAS
    protected String nazev, vzorec, synon;    // název, vzorec, synonyma látky
    public Vlastnost[] vlast = new Vlastnost[11];   // pole vlastností látky (T)
    // bodové vlastnosti
    protected double molHmot;         // mol. hmotnost (kg/mol)
/**
 * Kritická teplota (K)
 */    
    protected double tKr;             // krit. teplota (K)
/**
 * Kritický tlak (Pa)
 */    
    protected double pKr;             // krit. tlak (Pa)
/**
 * Kritický molární objem (m3/mol)
 */    
    protected double vKr;             // krit. mol. objem (m3/mol)
/**
 * Acentrický faktor, omega
 */    
    protected double acent;           // acentrický faktor, omega
    protected double gyr;             // gyrační poloměr
    protected double indLomu;         // index lomu
    protected double tIndLomu;        // teplota indexu lomu (K)
    protected double dipolm;          // dipolmoment
    protected double tDipolm;         // teplota dipolmomentu
    protected double dielektr;        // rel. ká permitivita, eps
    protected double tDielektr;       // teplota stat. permitivity (K)
/**
 * Teplo tání (J/mol)
 */    
    protected double dHtani;          // teplo tání (J/mol)
/**
 * Teplota tání (K)
 */    
    protected double tTani;           // teplota tání (K)
/**
 * Teplota normálního bodu varu (K)
 */    
    protected double tNbv;            // teplota norm. bodu varu (K)
    protected double rez1;            // rezerva
    protected double rez2;            // rezerva
    protected static List<Latka> latkaList;   // načtený seznam všech látek ********
    protected static int nZaznamu;    // počet načtených záznamů ze souboru
    protected static int[] indexZaznamu = new int[701];  // index záznamu pro číslo látky
     // cesta k souboru .drc s fyz. daty
    static String souborDat  = "";     // soubor z kterého byla načtena data látek 
    protected boolean hlidatMeze = true; // zapíná/vypíná hlídání mezí korel. vztahů
//    private static DataOutputStream os = null; 
    private static DataInputStream is = null; 
    private static int count = 0;


/**
 * Načtení údajů o všech látkách ze souboru <b>fydat.drc</b>
 * @return ArrayList typu <Latka> s údaji o látkách
 * @throws IOException
 */
    private static List<Latka> loadFromFile()  {
        List<Latka> list = new ArrayList();
        DataInputStream fr = null;
        if(indexZaznamu[2] == 0) Arrays.fill(indexZaznamu, -1);
        // naplnit seznam ze souboru s látkami
//        String fName = "d:/Java/Priklad2/src/ctiFyd/fydat.drc";
        try {
            fr = new DataInputStream(Latka.class.getResourceAsStream("data/Fydat.drc"));
    //        String fName = cestaBazSouboru;
    //        RandomAccessFile fr = new RandomAccessFile(fName, "r");
            int ktery, i;
            byte[] pomB20 = new byte[20];  // pomocné načítací pole
            byte[] pomB30 = new byte[30];   // pomocné načítací pole
            byte[] pomCAS = new byte[4];

    //  vlastní čtení souboru s přímýnm přístupem, délka záznamu 508 byte
    //        ktery = 1;     // číslo záznamu    ********************************
    //        String pom = JOptionPane.showInputDialog("Vlož číslo látky: ", "1");
    //        ktery = Integer.parseInt(pom);   //throws NumberFormatException;
    //        if (ktery > 0) {
    //            fr.seek((long) (ktery - 1) * 508);   // 508 je délka věty v bytech
    //        }
            nZaznamu=0;
            for (ktery = 0; ktery < 700; ktery++) {     // ---------- cyklus ----------
                Latka lt = new Latka();     // instance, přes kterou se načítá
    //            fr.seek((long) (ktery) * 508);   // 508 je délka věty v bytech
                lt.cislo = swapShort(fr.readShort());
                for (i = 0; i <= 29; i++) {
                    pomB30[i] = fr.readByte();
                    lt.nazev = new String(pomB30).trim();
                } // čte Název 30 zn.
                for (i = 0; i <= 19; i++) {
                    pomB20[i] = fr.readByte();
                    lt.vzorec = new String(pomB20).trim();
                }
                for (i = 0; i <= 29; i++) {
                    pomB30[i] = fr.readByte();
                    lt.synon = new String(pomB30).trim();
                }
    //            System.out.println("N= " + lt.cislo + " " + lt.nazev
    //                    + " " + lt.vzorec + " " + lt.synon);
                lt.group = fr.readShort() / 256;     // Převod hodnoty z horního byte
                lt.cas = swapInt(fr.readInt());
    //        System.out.println("Group= " + lt.group + " CAS: " + lt.cas);
                lt.molHmot = readFloatLittleEndian(fr);   //fr.readFloat();
                lt.tKr = readFloatLittleEndian(fr);
                lt.pKr = readFloatLittleEndian(fr);
                lt.vKr = readFloatLittleEndian(fr);
    //        System.out.println("Mv,Tkr,Pkr(MPa),Vkr: " + lt.molHmot + "  "
    //                + lt.tKr + "  " + (lt.pKr / 1E6) + "  " + lt.vKr);
                //System.out.println("Point1: " + fr.getFilePointer());
                lt.acent = readFloatLittleEndian(fr);
                lt.gyr = readFloatLittleEndian(fr);
                lt.indLomu = readFloatLittleEndian(fr);
                lt.tIndLomu = readFloatLittleEndian(fr);
                lt.dipolm = readFloatLittleEndian(fr);
                lt.tDipolm = readFloatLittleEndian(fr);
                lt.dielektr = readFloatLittleEndian(fr);
                lt.tDielektr = readFloatLittleEndian(fr);
                lt.dHtani = readFloatLittleEndian(fr);
                lt.tTani = readFloatLittleEndian(fr);
                lt.tNbv = readFloatLittleEndian(fr);
                lt.rez1 = readFloatLittleEndian(fr);
                lt.rez2 = readFloatLittleEndian(fr);
    //        System.out.format("dHt=%-9.5G Ttani(°C)=%-4.1f  Tnbv(°C)=%-4.1f%n ", lt.dHtani,
    //                (lt.tTani - 273.15), (lt.tNbv - 273.15));
                for (int k = 0; k < 11; k++) {
                    lt.vlast[k] = new Vlastnost();  // nutná inicializace
                    for (i = 0; i < 6; i++) {
                        lt.vlast[k].a[i] = readFloatLittleEndian(fr);
                    }
                    lt.vlast[k].vztah = swapShort(fr.readUnsignedShort());
                    lt.vlast[k].trida = swapShort(fr.readUnsignedShort());
                    lt.vlast[k].iOd = swapShort(fr.readUnsignedShort());
                    lt.vlast[k].iDo = swapShort(fr.readUnsignedShort());
                    for (i = 0; i <= 5; i++) {
    //                System.out.print("  A[" + i + "]= " + lt.vlast[k].a[i]);
                    }
    //            System.out.println(" ** iTyp,iAcc,iOd,iDo: " + lt.vlast[k].vztah
    //                    + " " + lt.vlast[k].trida + " " + lt.vlast[k].iOd + " " + lt.vlast[k].iDo);
                }
                //System.out.println();
    //        System.out.println("Pointer: " + fr.getFilePointer() + " " + fr.getFilePointer() / 508);
                if(!lt.nazev.equals("")|| lt.cislo == 350) {
                list.add(lt);     // přidat přečtenou látku do seznamu (jen neprázdné)
                indexZaznamu[lt.cislo]= nZaznamu;
                nZaznamu++;
                }
            }   // konec cyklu čtení for -------------------------------------
            fr.close();
        }
        catch (IOException e) {
            JOptionPane.showMessageDialog(null,
                "Chyba IOException při čtení souboru s daty látek!",
                "Čtení fydat.drc", JOptionPane.ERROR_MESSAGE); 
        }
        return list;
    } // konec loadFromFile
    
/**
 * Načtení údajů o všech látkách ze souboru <b>fydat.drc</b>
 * @return ArrayList typu <Latka> s údaji o látkách
 * @throws IOException
 */
    private static List<Latka> loadFromXmlFile(File dataFile)  {
      List<Latka> list = new ArrayList();
      if(indexZaznamu[2] == 0 && latkaList == null) Arrays.fill(indexZaznamu, -1);
      Document document = null;  
      Element rozvoje; // teplotní rozvoje
      Element vlastnost; // teplotní vlastnost         
     SAXBuilder saxBuilder = new SAXBuilder();     
     if(dataFile == null) dataFile = new File("fydat.xml"); 
     boolean ex = dataFile.exists();
  try {  
   // converted file to document object  
   if(ex)document = saxBuilder.build(dataFile);      
   if(!ex)document = saxBuilder.build(is);      
   // get root node from xml  
   Element rootNode = document.getRootElement();      
   // got all xml elements into a list  
      List<Element> xmlList = rootNode.getChildren("latka");  
      if(!"fydat".equals(rootNode.getName()) || !"latka".equals(xmlList.get(0).getName())) {
            JOptionPane.showMessageDialog(null,
                "Chyba, datový soubor fydat.xml či  fydat.zip nemá správnou strukturu! \n"
                        + "Odstraňte ho z adresáře. Teď se použije interní báze.\nPokračuji!",
                "FYDAT - Databáze látek", JOptionPane.ERROR_MESSAGE);
            return null;
//         System.exit(0);   
      }  
      if(latkaList == null) nZaznamu = 0;  // od nuly jen při načtení základdních dat 
      for(int i = 0;i <= xmlList.size()-1;i++){
       Latka lt = new Latka();  // objekt Latka do kterého se načítá
       Element element = xmlList.get(i);
       lt.cislo = Integer.parseInt(element.getAttribute("id").getValue());
       lt.nazev = element.getChildText("nazev");
//       System.out.println("Látka : "+lt.cislo+"  "+lt.nazev);
       lt.vzorec = element.getChildText("vzorec");
//       System.out.println("Vzorec : "+element.getChildText("vzorec"));
       lt.synon = element.getChildText("synon");
       lt.group = Integer.parseInt(element.getChildText("group").replaceAll(",", ""));
       lt.cas = Long.parseLong(element.getChildText("cas"));
       Element elbodvl = element.getChild("bodvlast");
       lt.molHmot = prevedNaDouble(elbodvl.getChildText("molhmot"));
       lt.tKr = prevedNaDouble(elbodvl.getChildText("tkr"));
       lt.pKr = prevedNaDouble(elbodvl.getChildText("pkr"));
       lt.vKr = prevedNaDouble(elbodvl.getChildText("vkr"));
       lt.acent = prevedNaDouble(elbodvl.getChildText("acent"));
       lt.gyr = prevedNaDouble(elbodvl.getChildText("gyr"));
       lt.indLomu = prevedNaDouble(elbodvl.getChildText("indLomu"));
       lt.tIndLomu = prevedNaDouble(elbodvl.getChildText("tindLomu"));
       lt.dielektr = prevedNaDouble(elbodvl.getChildText("dielektr"));
       lt.tDielektr = prevedNaDouble(elbodvl.getChildText("tdielektr"));
       lt.dHtani = prevedNaDouble(elbodvl.getChildText("dHtani"));
       lt.tTani = prevedNaDouble(elbodvl.getChildText("tTani"));
       lt.tNbv = prevedNaDouble(elbodvl.getChildText("tnbv"));
       lt.rez1 = prevedNaDouble(elbodvl.getChildText("rezerva1"));
       lt.rez2 = prevedNaDouble(elbodvl.getChildText("rezerva2"));
       rozvoje = element.getChild("rozvoje");
       List<Element> xmlVlast = rozvoje.getChildren("vlastnost");
       for(int j = 0; j < 11; j++) {
           lt.vlast[j] = new Vlastnost();
           vlastnost = xmlVlast.get(j);
           lt.vlast[j].a[0] = prevedNaDouble(vlastnost.getChildText("a0"));
           lt.vlast[j].a[1] = prevedNaDouble(vlastnost.getChildText("a1"));
           lt.vlast[j].a[2] = prevedNaDouble(vlastnost.getChildText("a2"));
           lt.vlast[j].a[3] = prevedNaDouble(vlastnost.getChildText("a3"));
           lt.vlast[j].a[4] = prevedNaDouble(vlastnost.getChildText("a4"));
           lt.vlast[j].a[5] = prevedNaDouble(vlastnost.getChildText("a5"));
           lt.vlast[j].vztah = Integer.parseInt(vlastnost.getChildText("vztah"));
           lt.vlast[j].trida = Integer.parseInt(vlastnost.getChildText("trida"));
           lt.vlast[j].iOd = Integer.parseInt(vlastnost.getChildText("iOd"));
           lt.vlast[j].iDo = Integer.parseInt(vlastnost.getChildText("iDo"));
//           System.out.println("* "+j + " " +vlastnost.getAttributeValue("typ"));
       }
       list.add(lt);   // přidá načtenou látku do seznamu všech látek
       indexZaznamu[lt.cislo]= nZaznamu;    // pro číslo látky vrací index záznamu
       nZaznamu++;
      }  
       
  } catch (JDOMException e) {  
        JOptionPane.showMessageDialog(null,
                "Nepodařilo se načíst soubor s daty látek!\n" +
                "Zřejmě chybná struktura fydat.xml.\n" +
                "Odstraňte ho z adresáře. Teď se použije interní báze.\nPokračuji!",
                "FYDAT - Čtení fydat.xml", JOptionPane.ERROR_MESSAGE);
        return null;
//        System.exit(0);  
  } catch (IOException e) {  
        JOptionPane.showMessageDialog(null,
                "Chyba IOException při čtení souboru s daty látek!",
                "Čtení fydat.xml", JOptionPane.ERROR_MESSAGE);  
  }         
        return list;
  }
/**
 * Převede textový string na double
 * Nahradí des. čárku tečkou
 * @param tx
 * @return 
 */    
    private static double prevedNaDouble(String tx) {
        if(tx == null) return -9999.0;
        tx = tx.replace(",", ".");
        if(tx.equals("")) return 0.0;
        return Double.parseDouble(tx);
    }
/**
 * Zajistí načtení dat o látkách do seznamu latkaList
 * Existuje-li v adresáři aplikace fyadt.xml, použije se tento,
 * jinak fydat.drc z resource
 */    
    private static void nactiDataLatek() {
        if (latkaList == null) {
            File xmlFile = new File("fydat.xml");
            if(xmlFile.exists()) {
                latkaList = loadFromXmlFile(null);  // fydat.xml z adresáře
                souborDat = xmlFile.getName();
            } else if((new File("fydat.zip")).exists()) {   // ale existuje .zip
                unZip("fydat.zip");
                latkaList = loadFromXmlFile(null); // fydat.drc z resource
                souborDat = "fydat.zip";  
                try {
                    is.close();
                } catch (IOException ex) {
                    Logger.getLogger(Latka.class.getName()).log(Level.SEVERE, null, ex);
                }
            } else {
                latkaList = loadFromFile(); // fydat.drc z resource
                souborDat = "fydat.drc  (interní)";
            }
        if (latkaList == null) { // nepovedlo se předchozí čtení xml, zip
                latkaList = loadFromFile(); // fydat.drc nakonec z resource
                souborDat = "fydat.drc  (interní)";            
        }           
        }
    }
/**
 * Extrakce zip souboru bez využití třídy ZipFile do DataInputStreamu pro  xml
 * @param jmenoSouboru zip soubor
 */      
    public static void unZip(String jmenoSouboru) {
      try {
         BufferedOutputStream dest = null;
         FileInputStream fis = new FileInputStream(jmenoSouboru);
         ZipInputStream zis = new ZipInputStream(new BufferedInputStream(fis));
         ZipEntry entry;
         entry = zis.getNextEntry();
               is = new DataInputStream(zis);               
      } catch(IOException e) {
            JOptionPane.showMessageDialog(null,
                "ZIP " + jmenoSouboru + " se nepodařilo dekodovat!",
                 "UnZip", JOptionPane.ERROR_MESSAGE);         
      }        
    }    
    
/**
 * Vybrání látky podle jejího čísla
 * @param id skutečné číslo látky 
 * @return Vrací látku daného čísla
 */
    public static Latka getLatka(int id)  {
        Latka lt;
        if (latkaList == null) {
            nactiDataLatek();
        }
        try {
            count = 0;
//            int index = najdiIndex(latkaList, id, 0, latkaList.size()-1);
            return latkaList.get(indexZaznamu[id]);
        }
        catch (IndexOutOfBoundsException e) {
            return null;
        }
    }
///**
// * Interpolační vyhledávání v seznamu latkaList rekursivně (není třeba)
// * @param latkaList
// * @param id hledané číslo látky
// * @param from  dolní mez
// * @param to    horní mez
// * @return 
// */    
//    private static int najdiIndex(List<Latka> latkaList, int id, int from, int to) {
//
//        if(latkaList.get(0).cislo == id) return from;  //nalezli jsme, treba kontrolovat kvuli deleni nulou dale v algoritmu
////        else if(from == to || array[from] == array[to]) return -1; //zarazka rekurze, nehrozí
//        //provedeme odhad pozice hledaneho cisla na zaklade soucasneho rozsahu pozic, hodnot mezi a hledane hodnoty
//        int index = from + (int)(((float)(to - from)/(float)(latkaList.get(to).cislo - latkaList.get(from).cislo)) * (float)(id - latkaList.get(from).cislo));
//        count++;
////        System.out.println("i: "+count+"  "+index);
//        if(count > 40) {
//            JOptionPane.showMessageDialog(null,
//                "Chyba, látka číslo: " + id + " není v databázi!",
//                "Načítání látky", JOptionPane.ERROR_MESSAGE);             
//            return -9999;   // vyvolá chybu výšee
//        }
//        if(latkaList.get(index).cislo == id) return index;//cislo jsme nalezli
//        //nenalezli jsme, pokracujeme v prave polovine, pokud je hledane cislo vyssi nez predel
//        else if(latkaList.get(index).cislo < id) return najdiIndex(latkaList, id, index + 1, to);
//        //nenalezli jsme, pokracujeme v leve polovine, pokud je hledane cislo nizsi nez predel
//        else return najdiIndex(latkaList, id, from, index - 1);
//
//    }
    
/**
 * Vrací seznam všech látek v databázi jako seznam objektů Latka
 * @return seznam látek <Latka>
 */    
    public static List<Latka> getLatkaList() {
        if (latkaList == null) 
            nactiDataLatek();       
        return latkaList;
    }    
    
/**
 * Výpočet vlastnosti látky z korelačních vztahů
 * @param t  - absolutní teplota (K)
 * @param nVl - číslo teplotně závislé vlastnosti
 * @return  hodnota vlastnosti
 */
    private double ffce(double t, int nVl) {
        // t absolutní teplota (K),  nVl  poř. číslo vlastnosti
        // vyčíslí nVl-tou teplotně závislou vlastnost látky
        // omezení na podkritickou oblast
        int ityp;   // typ korelačního vztahu
        double tt;  // pomocné redukované teploty
        double ff = 0, pom;

        if (t > tKr) {  // je teplota nadkritická?
            t = tKr;
        }
        // Pokud je teplota mimo meze vztahu, dosaď mezní hodnotu
        if (hlidatMeze && t > vlast[nVl].iDo) {
            t = vlast[nVl].iDo;
        }
        if (hlidatMeze && t < vlast[nVl].iOd) {
            t = vlast[nVl].iOd;
        }

        // hustota vody přesněji
        if (nVl == 3 && cislo == 1){
            tt=1-t/647.096;
            ff=1.99274064*Math.pow(tt,(double)1/3)+1.09965342*Math.pow(tt,(double)2/3)-0.510839303*Math.pow(tt,(double)5/3);
            pom=-1.7549379*Math.pow(tt,(double)16/3)-45.5170352*Math.pow(tt,(double)43/3)-674694.45*Math.pow(tt,(double)110/3);
            ff=322*(1+ff+pom);
            return ff;
        }
        ityp = vlast[nVl].vztah - 1;  // typ korelačního vztahu pro javu
        switch (ityp) {
            case 0:     // přímka
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] * t;
                break;
            case 1:     // parabola
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] * t
                        + vlast[nVl].a[2] * t * t;
                break;
            case 2:     // pol. 3. stupně
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] * t
                        + vlast[nVl].a[2] * t * t
                        + vlast[nVl].a[3] * t * t * t;
                break;
            case 3:
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] * t
                        + vlast[nVl].a[2] * t * t
                        + vlast[nVl].a[3] * t * t * t
                        + vlast[nVl].a[4] * t * t * t * t;
                break;
            case 4:
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] * t
                        + vlast[nVl].a[2] * t * t
                        + vlast[nVl].a[3] * t * t * t
                        + vlast[nVl].a[4] * t * t * t * t
                        + vlast[nVl].a[5] * t * t * t * t * t;
                break;
            case 5:     // Antoin
                ff = Math.exp(vlast[nVl].a[0] + vlast[nVl].a[1] / (vlast[nVl].a[2] + t));
                break;
            case 6:     // Hustota kapalin
                ff = 3 + Math.log10(vlast[nVl].a[0]) - Math.pow((1 - t / vlast[nVl].a[2]), 0.285714) * Math.log10(vlast[nVl].a[1]);
                ff = Math.pow(10, ff);
                break;
            case 7:     // dHvyp, sigma
                ff = vlast[nVl].a[0] * Math.pow((vlast[nVl].a[3] - t) / (vlast[nVl].a[3] - vlast[nVl].a[1]), vlast[nVl].a[2]);
                break;
            case 8:     // tenze par
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] / (vlast[nVl].a[2] + t) + vlast[nVl].a[3] * Math.log10(t) + vlast[nVl].a[4] * t + vlast[nVl].a[5] * t * t;
                ff = Math.pow(10, ff);
                break;
            case 9:     // Visk 
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] / (vlast[nVl].a[4] + t) + vlast[nVl].a[2] * t + vlast[nVl].a[3] * t * t;
                ff = Math.pow(10, ff);
                break;
            case 10:     // Visk L
                ff = Math.exp(vlast[nVl].a[0] + vlast[nVl].a[1] / t + vlast[nVl].a[2] / t / t + vlast[nVl].a[3] / t / t / t);
                break;
            case 11:     // Visk G
                ff = vlast[nVl].a[0] * Math.pow(t, 1.5) / (t + vlast[nVl].a[1]);
                break;
            case 12:     // Wagner, tenze
                tt = 1 - t / vlast[nVl].a[4];
                ff = (vlast[nVl].a[0] * tt + vlast[nVl].a[1] * Math.pow(tt, 1.5) + vlast[nVl].a[2] * tt * tt * tt + vlast[nVl].a[3] * tt * tt * tt * tt * tt * tt) / t * vlast[nVl].a[4];
                ff = Math.exp(ff) * vlast[nVl].a[5];
                break;
            case 13:     // vodivost (l)
                tt = 1 - t / vlast[nVl].a[1];
                ff = vlast[nVl].a[0] * Math.pow((1 - tt), 0.38) / Math.pow(tt, 1 / 6);
                break;
            case 14:     // hust (l) CH
                ff = vlast[nVl].a[0] / Math.pow(vlast[nVl].a[1], (1 + Math.pow((1 - t / vlast[nVl].a[2]), vlast[nVl].a[3])));
                break;
            case 15:     // dHv, sigma
                tt = 1 - t / vlast[nVl].a[5];
                ff = vlast[nVl].a[0] * Math.pow((1 - tt), vlast[nVl].a[1] + vlast[nVl].a[2] * tt + vlast[nVl].a[3] * tt * tt);
                break;
            case 16:     // Visk L
                ff = vlast[nVl].a[0] * (1 / t - 1 / vlast[nVl].a[1]) + vlast[nVl].a[2];
                ff = Math.pow(10, ff);
                break;
            case 17:     // Visk L
                ff = vlast[nVl].a[0] + vlast[nVl].a[1] / t + vlast[nVl].a[2] * Math.log(t) + vlast[nVl].a[3] * Math.pow(t, vlast[nVl].a[4]);
                ff = Math.exp(ff);
                break;
            case 18:     // Visk G
                ff = vlast[nVl].a[0] * Math.pow(t, vlast[nVl].a[1]) / (1 + vlast[nVl].a[2] / t + vlast[nVl].a[3] / t / t);
                break;
        }
        return ff;
    }   // konec ffce

    private static int swapShort(int i) {  // čte short (Fortran integer) do int
        return ((i & 0xff00) >>> 8) + ((i & 0x00ff) << 8);
    }

    private static int swapInt(int i) {
        int k;  // čte integer (Fortran int*4) a převede ho do ing
        k = ((i & 0x000000ff) << 24) + ((i & 0x0000ff00) << 8)
                + ((i & 0x00ff0000) >>> 8) + ((i & 0xff000000) >>> 24);
        return k;
    }

    static double readFloatLittleEndian(DataInputStream fr) throws IOException {
        // get 4 unsigned raw byte components, accumulating to an int,
        // then convert the 32-bit pattern to a float.
        int accum = 0;
        for (int shiftBy = 0; shiftBy < 32; shiftBy += 8) {
            accum |= (fr.readByte() & 0xff) << shiftBy;
        }
        return Float.intBitsToFloat(accum);
    }
//    static double readFloatLittleEndian(RandomAccessFile fr) throws IOException {
//        // get 4 unsigned raw byte components, accumulating to an int,
//        // then convert the 32-bit pattern to a float.
//        int accum = 0;
//        for (int shiftBy = 0; shiftBy < 32; shiftBy += 8) {
//            accum |= (fr.readByte() & 0xff) << shiftBy;
//        }
//        return Float.intBitsToFloat(accum);
//    }
/**
 * Seznam látek do stringu
 * v pořadí, jak jsou v seznamu <b>latkaList</b>
 * @return seznam všech látek (látka na řádce)
 * @throws IOException 
 */
    public static String seznamLatek()  {
        String sezn="";
        Latka lt = new Latka();
        int i, n;
        if (nZaznamu == 0) Latka.getLatka(0);    // prvotní načtení
        n = 0;    // počítadlo neprázdných látek v bázi
        for (i = 0; i < nZaznamu; i++) {
            lt = latkaList.get(i);
            if (lt.getNazev().equals("") ) continue;
//            System.out.println("* " + lt.getCislo() + " " + lt.getNazev()
//                    + " " + lt.getVzorec() + " " + lt.getSynon());
//            System.out.format("* %-3d %-30s  %-10s %-30s%n",lt.getCislo(),lt.getNazev(),
//                    lt.getVzorec(),lt.getSynon());
            sezn += String.format("* %-3d %-30s  %-10s %-30s%n",lt.getCislo(),lt.getNazev(),
                    lt.getVzorec(),lt.getSynon());
            n++;
        }
        return sezn;
    }
/**
 * Seznam látek do stringového pole s
 * @param abecedne
 * @param  String[] s
 * @param  boolean true => abecedne; false => podle čísel
 * @return setříděné String[] s
 * @throws IOException
 */
    public static  String[] seznamLatek( boolean abecedne)   {
        Latka lt = new Latka();
        int i,n;
        if(nZaznamu == 0) Latka.getLatka(0);    // prvotní načtení
        n = 0;    // počítadlo neprázdných látek v bázi
        for (i = 0; i < nZaznamu; i++) {
            lt = latkaList.get(i);
            if (lt.getNazev().equals("") ) continue;
            n++;    // zjištění počtu nenulových záznamů
        }
        String[] s = new String[n];
        n = 0;
        for (i = 0; i < nZaznamu; i++) {
            lt = latkaList.get(i);
            if (lt.getNazev().equals("") ) continue;
            s[n]= String.format("%3d %-30s %-10s %-30s%n",lt.getCislo(),lt.getNazev(),
                    lt.getVzorec(),lt.getSynon());  // normální obsah
//            s[n]= String.format("%-30s %-10s %-30s %3d%n",lt.getNazev(),
//                    lt.getVzorec(),lt.getSynon(),lt.getCislo());  // číslo na konci !
            n++;
        }
//        Arrays.sort(s, new KomparSeznam());
        if (abecedne) {
        Arrays.sort( s, new Comparator<String>() {  // komparátor jako vnitřní třída
            public int compare(String o1, String o2) {
                if( o1 == null ) return -1;
                return o1.substring(4).compareTo(o2.substring(4));
            }
        });
        } else {
        Arrays.sort( s, new Comparator<String>() {  // komparátor jako vnitřní třída
            public int compare(String o1, String o2) {
                if( o1 == null ) return -1;
                return o1.substring(0,3).compareTo(o2.substring(0,3));
            }
        });            
        }


        return s;
    }
    
/**
 * Výběr souboru s doplňkovými datay o látkách
 * @return vybraný soubor nebo null
 */    
    public static void nahrajDodatecneLatky() {
        // vyhledání soubory s vlastními dodatkovými látkami
        JFileChooser fc = new JFileChooser("c:/Data/Java/CHIDat");
//        JFileChooser fc = new JFileChooser();
        File dataFile = null;        
        fc.setDialogTitle("Výběr dodatečných dat");
        fc.setFileFilter(new FileNameExtensionFilter("Soubory XML", "xml","XML"));
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        int res = fc.showOpenDialog(null);
        if (res != JFileChooser.APPROVE_OPTION) {
            JOptionPane.showMessageDialog(null,
                    "Zrušení vyhledání nebo chyba při vyhledávání souboru!",
                    "FYDAT dodatečné látky", JOptionPane.ERROR_MESSAGE);            
            return;
        }
            dataFile = fc.getSelectedFile();
//            adrZdroj = dataFile.getPath();
        if(!dataFile.exists()) {
            JOptionPane.showMessageDialog(null,
                    "Soubor " + dataFile + " nenalezen!",
                    "FYDAT dodatečné látky", JOptionPane.WARNING_MESSAGE);            
            return;            
        }    
        // soubor xml existuje načti z něj látky a přidej je do latkaList 
//        loadFromXmlFile(dataFile);
          boolean suc = latkaList.addAll(loadFromXmlFile(dataFile));  // doplní dodatkové látky
//          System.out.println("Dopln: "+suc); 
    }    

    //  Gettery a settery , bodové vlastnosti látky
    
/**
 * Vrací pole s indexyZaznamu pro čísla látek
 * @return 
 */    
    public static int[] getIndexZaznamu() {
        return indexZaznamu;
    }
    
/**
 * Vrací hodnotu čísla látky
 * @return číslo látky
 */
    public int getCislo() {
        return cislo;
    }
/**
 * Vrací počet záznamů látek
 * @return 
 */
    public static int getnZaznamu() {
        return nZaznamu;
    }   
    
/**
 * Vrací molekulární hmotnost látky (kg/mol)
 * @return molární hmotnost (kg/mol)
 */
    public double getMolHmot() {
        return molHmot;
    }
/**
 * Vrací název látky
 * @return String název
 */
    public String getNazev() {
        return nazev;
    }
/**
 * Vrací kritický tlak látky (Pa)
 * @return pKr
 */
    public double getpKr() {
        return pKr;
    }
/**
 * Vrací synonyma názvu látky
 * @return String synon
 */
    public String getSynon() {
        return synon;
    }
/**
 * Vrací kritickou teplotu látky (K)
 * @return teplota (K)
 */
    public double gettKr() {
        return tKr;
    }
/**
 * Vrací teplotu normálního bodu varu (K)
 * @return teplota (K)
 */
    public double gettNbv() {
        return tNbv;
    }
/**
 * Vrací hodnotu teploty tání látky (T)
 * @return teplota (K)
 */
    public double gettTani() {
        return tTani;
    }
/**
 * Vrací hodnotu kritického objemu látky (m3/mol)
 * @return vKr
 */
    public double getvKr() {
        return vKr;
    }
/**
 * Vrací sumární vzorec látky
 * @return vzorec jako String
 */
    public String getVzorec() {
        return vzorec;
    }
/**
 * Vrací hodnotu acentrického faktoru látky
 * @return
 */
    public double getAcent() {
        return acent;
    }
/**
 * Vrací číslo CAS látky
 * @return CAS
 */
    public long getCas() {
        return cas;
    }
/**
 * Vrací teplo tání látky (J/mol)
 * @return dHtani
 */
    public double getdHtani() {
        return dHtani;
    }
/**
 * Vrací hodnotu teploty pro dielektrickou konstantu látky
 * @return eps
 */
    public double getDielektr() {
        return tDielektr;
    }
/**
 * Vrací hodnotu dielektrické konstanty látky
 * @return eps
 */
    public double gettDielektr() {
        return tDielektr;
    }
    
/**
 * Vrací hodnotu dipolmomentu látky
 * @return dipolmoment
 */
    public double getDipolm() {
        return dipolm;
    }
/**
 * Vrací skupinu typu látky
 * @return group
 */
    public int getGroup() {
        return group;
    }
/**
 * Vrací hodnotu gyračního poloměru látky
 * @return gyr
 */
    public double getGyr() {
        return gyr;
    }
/**
 * Vrací hodnotu indexu lomu kapaliny
 * @return nL
 */
    public double getIndLomu() {
        return indLomu;
    }
/**
 * Vrací hodnotu teploty indexu lomu (°C)
 * @return Tindlomu (°C)
 */
    public double gettIndLomu() {
        return tIndLomu - 273.15;
    }
/**
 * Umožní operativně změnit název látky
 * @param nazev látky
 */
    public void setNazev(String nazev) {
        this.nazev = nazev;
    }
/**
 * Vrací číslo typu korelačního vzathu
 * @param int ii  typ vztahu
 * @return typ vztahu
 */
    public int getVztah(int ii) {
        return vlast[ii].vztah;
    }
/**
 * Vrací třídu přesnosti korel.vztahu (záporně náhradní látka pro vlasynost)
 * @param int ii  Třída
 * @return tr
 */
    public int getTrida(int ii) {
        return vlast[ii].trida;
    }
/**
 * Vrací dolní mez platnosti korelačního vztahu (K)
 * @param ii-tá vlastnost
 * @return tOd
 */
    public int getiOd(int ii) {
        return vlast[ii].iOd;
    }
/**
 * Vrací horní mez platnosti korelačního vztahu (K)
 * @param ii-tá vlastnost 
 * @return tDo
 */
    public int getiDo(int ii) {
        return vlast[ii].iDo;
    }

/**
 * Zapíná hlídání mezí korelačních vztahů látek
 * @param hlidatMeze
 */
    public void setHlidatMeze(boolean hlidatMeze) {
        this.hlidatMeze = hlidatMeze;
    }
/**
 * Vrací stav hlídání mezí korelačních vztahů látek
 * @param hlidatMeze  true/false
 */

    public boolean isHlidatMeze() {
        return hlidatMeze;
    }
/**
 * Vrací hodnotu dipolmomentu látky
 * @return 
 */
    public double gettDipolm() {
        return tDipolm;
    }
/**
 * Vrací název souboru, zkterého byla načtena dat ao látách
 * @return 
 */
    public static String getSouborDat() {
        return souborDat;
    }


//  Teplotně závislé fyz. vlastnosti látky
/**
 * Vrací viskozitu par látky (Pa.s)
 * @param t abs. teplota (K)
 * @return visk
 */
    public double viskG(double t) {
        return ffce(t, 0);
    }
/**
 * Vrací izobarickou tepelnou kapacitu par látky
 * @param t abs. teplota (K)
 * @return Cp (J/mol/K)
 */
    public double cpG(double t) {
        return ffce(t, 1);
    }
/**
 * Vrací hodnotu tepelné vodivosti par látky
 * @param t abs. teplota (K)
 * @return  vodivost (W/m/K)
 */
    public double vodG(double t) {
        return ffce(t, 2);
    }
/**
 * Vrací hodnoty hustoty kapalné fáze látky
 * @param t abs. teplota (K)
 * @return  hust (kg/m3)
 */
    public double hustL(double t) {
        return ffce(t, 3);
    }
/**
 * Vrací viskozitu kapalné látky (Pa.s)
 * @param t abs. teplota (K)
 * @return visk
 */
    public double viskL(double t) {
        return ffce(t, 4);
    }
/**
 * Vrací hodnotu tepelné vodivosti kapaliny
 * @param t abs. teplota (K)
 * @return  vodivost (W/m/K)
 */
    public double vodL(double t) {
        return ffce(t, 5);
    }
/**
 * Vrací izobarickou tepelnou kapacitu kapaliny
 * @param t abs. teplota (K)
 * @return Cp (J/mol/K)
 */
    public double cpL(double t) {
        return ffce(t, 6);
    }
/**
 * Vrací hodnotu výparného tepla látky
 * @param t abs. teplota (K)
 * @return dHv  (J/mol)
 */
    public double dHv(double t) {
        return ffce(t, 7);
    }
/**
 * Vrací hodnotu tenze par látky
 * @param t abs. teplota
 * @return  tenze (Pa)
 */
    public double tenze(double t) {
        return ffce(t, 8);
    }
/**
 * Vrací hodnotu povrchového napětí látky
 * @param t abs. teplota (K)
 * @return  sigma (N/m)
 */
    public double sigma(double t) {
        return ffce(t, 9);
    }
}
